@@ -11,7 +11,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details. 
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -45,6 +45,8 @@
 #include "filter.h"
 #include "dft.h"
 #include "font.h"
+#include "joy.h"
+#include "mouse.h"
 
 #ifdef USE_LCD
 #include "lcd.h"
@@ -61,7 +63,7 @@
 #if defined(MESS)
 
 #define ADV_COPY \
-	"AdvanceMESS - Copyright (C) 2001-2003 by Andrea Mazzoleni\n" \
+	"AdvanceMESS - Copyright (C) 2001-2018 by Andrea Mazzoleni\n" \
 	"MESS - Copyright (C) 1998-2003 by the MESS Team\n" \
 	"MAME - Copyright (C) 1997-2003 by Nicola Salmoria and the MAME Team\n"
 
@@ -72,7 +74,7 @@
 #else
 
 #define ADV_COPY \
-	"AdvanceMAME - Copyright (C) 1999-2003 by Andrea Mazzoleni\n" \
+	"AdvanceMAME - Copyright (C) 1999-2018 by Andrea Mazzoleni\n" \
 	"MAME - Copyright (C) 1997-2003 by Nicola Salmoria and the MAME Team\n"
 
 #define ADV_TITLE "AdvanceMAME"
@@ -125,7 +127,7 @@ struct safequit_entry {
 struct advance_safequit_config_context {
 	char file_buffer[FILE_MAXPATH]; /**< File safequit.dat to load. */
 	adv_bool debug_flag; /**< Show the debug flag on the screen. */
-        adv_bool safe_exit_flag; /**< Flag for safe exit. */
+	adv_bool safe_exit_flag; /**< Flag for safe exit. */
 };
 
 struct advance_safequit_state_context {
@@ -155,20 +157,20 @@ void advance_safequit_update(struct advance_safequit_context* context);
 
 /** Max supported input devices. */
 /*@{*/
-#define INPUT_PLAYER_MAX 4 /**< Max numer of player. */
+#define INPUT_PLAYER_MAX 8 /**< Max numer of player. */
 
 #define INPUT_ANALOG_MAX 16 /**< Max number of analog controls for player. */
-#define INPUT_DIGITAL_MAX 4096 /**< Max number of digital ports definition. */
+#define INPUT_DIGITAL_MAX 16384 /**< Max number of digital ports definition. */
 
 #define INPUT_MAP_MAX 16 /**< Max number of mapping codes. */
 
-#define INPUT_KEYBOARD_MAX 4 /**< Max number of keyboard. */
+#define INPUT_KEYBOARD_MAX 8 /**< Max number of keyboard. */
 #define INPUT_JOY_MAX 8 /**< Max number of joysticks. */
+#define INPUT_MOUSE_MAX 8 /**< Max number of mouses. */
+
 #define INPUT_STICK_MAX 8 /**< Max number of sticks for a joystick. */
 #define INPUT_AXE_MAX 8 /**< Max number of axes for a stick or mouse. */
 #define INPUT_DIR_MAX 2 /**< Max number of direction for an axe (up/down or left/right). */
-#define INPUT_MOUSE_MAX 8 /**< Max number of mouses. */
-#define INPUT_BUTTON_MAX 64 /**< Max number buttons for a joystick or mouses. */
 
 #define INPUT_HELP_MAX 512 /**< Max number of help entry. */
 /*@}*/
@@ -208,13 +210,17 @@ struct advance_input_state_context {
 	unsigned char key_old[INPUT_KEYBOARD_MAX][KEYB_MAX]; /**< Keyboard previous frame state. */
 	unsigned char key_current[INPUT_KEYBOARD_MAX][KEYB_MAX]; /**< Keyboard current frame state. */
 
-	int joystick_button_current[INPUT_JOY_MAX][INPUT_BUTTON_MAX]; /**< Joystick button state. */
+	int joystick_button_current[INPUT_JOY_MAX][JOYB_MAX]; /**< Joystick button state. */
 	int joystick_analog_current[INPUT_JOY_MAX][INPUT_STICK_MAX][INPUT_AXE_MAX]; /**< Joystick analog state. */
 	int joystick_digital_current[INPUT_JOY_MAX][INPUT_STICK_MAX][INPUT_AXE_MAX][INPUT_DIR_MAX]; /**< Joystick digital state. */
 	int ball_analog_current[INPUT_JOY_MAX][INPUT_AXE_MAX]; /**< Joystick ball analog state. */
 
-	int mouse_button_current[INPUT_MOUSE_MAX][INPUT_BUTTON_MAX]; /**< Mouse button state. */
+	int mouse_button_current[INPUT_MOUSE_MAX][MOUSEB_MAX]; /**< Mouse button state. */
 	int mouse_analog_current[INPUT_MOUSE_MAX][INPUT_AXE_MAX]; /**< Mouse analog state. */
+
+	int simulate_key0; /**< Key code to simulate. 0 for none. */
+	int simulate_key1; /**< Key code to simulate. 0 for none. */
+	int simulate_counter; /**< Number of frames to simulate the key. */
 };
 
 struct advance_input_context {
@@ -236,7 +242,7 @@ adv_error advance_input_parse_analogname(unsigned* type, const char* buffer);
 adv_error advance_input_parse_analogvalue(int* delta, int* sensitivity, int* reverse, int* centerdelta, char* buffer);
 void advance_input_print_analogvalue(char* buffer, unsigned buffer_size, int delta, int sensitivity, int reverse, int center);
 adv_bool advance_input_digital_pressed(struct advance_input_context* context, unsigned code);
-
+void advance_input_simulate_key(struct advance_input_context* context, int key0, int key1, int frames);
 
 /***************************************************************************/
 /* UI */
@@ -611,9 +617,10 @@ void advance_sound_config_save(struct advance_sound_context* context, const char
 #define COMBINE_MAXMIN 1
 #define COMBINE_MEAN 2
 #define COMBINE_FILTER 3
-#define COMBINE_SCALE 6
-#define COMBINE_LQ 7
+#define COMBINE_SCALEX 6
+#define COMBINE_SCALEK 7
 #define COMBINE_HQ 8
+#define COMBINE_XBR 9
 /*@}*/
 
 /** Special additional effect (enumeration). */
@@ -668,7 +675,8 @@ struct advance_video_config_context {
 	unsigned blit_orientation; /**< Blit orientation mask. Mask of ORIENTATION_*. */
 	unsigned user_orientation; /**< User orientation mask. Mask of ORIENTATION_*. */
 	unsigned game_orientation; /**< Game orientation mask. Mask of ORIENTATION_*. */
-	int combine; /**< Special combine effect. Mask of COMBINE_*. */
+	int combine; /**< Combine effect. Mask of COMBINE_*. */
+	int combine_max; /**< Maximum combine effect. Always starting with COMBINE_XBR and then decreasing at runtime. */
 	int rgb_effect; /**< Special additional effect. Mask of EFFECT_*. */
 	int interlace_effect; /**< Special additional interlace effect. Mask of EFFECT_*. */
 	double turbo_speed_factor; /**< Speed of the turbo function. Multiplicative factor. */
@@ -677,7 +685,8 @@ struct advance_video_config_context {
 	int fastest_time; /**< Time for turbo at the startup [seconds]. */
 	int measure_time; /**< Time for the speed measure [seconds]. */
 	adv_bool restore_flag; /**< Reset the video mode at the exit [boolean]. */
-	unsigned magnify_factor; /**< Magnify factor requested [1,2,4]. */
+	unsigned magnify_factor; /**< Magnify factor requested [0=auto,1,2,3,4]. */
+	unsigned magnify_size; /**< Magnify target size. */
 	unsigned index; /**< Index mode. */
 	double frameskip_factor; /**< Current frameskip factor. */
 	adv_bool frameskip_auto_flag; /**< boolean. */
@@ -709,7 +718,7 @@ struct advance_video_config_context {
 #define AUDIOVIDEO_MEASURE_MAX 17
 
 #define PIPELINE_MEASURE_MAX 13
-#define PIPELINE_BLIT_MAX 3
+#define PIPELINE_BLIT_MAX 2 /**< Number of pipelines to create. 0 for buffered, 1 for direct write. */
 
 /** State for the video part. */
 struct advance_video_state_context {
@@ -784,6 +793,8 @@ struct advance_video_state_context {
 	unsigned skip_level_skip; /**< Number of frames to skip in the cycle. */
 	unsigned skip_level_sum; /**< Total number of frame in the cycle. */
 	adv_bool skip_level_disable_flag; /**< If skipping was disabled for some reasons. */
+	unsigned skip_level_combine_counter; /**< Counter of slow frame before decreasing the combine effect. */
+	unsigned skip_level_combine_total; /**< Total counter of slow frame before decreasing the combine effect. */
 
 	int latency_diff; /**< Current sound latency error in samples. */
 
@@ -867,6 +878,14 @@ struct advance_video_state_context {
 	unsigned pipeline_measure_i;
 	unsigned pipeline_measure_j;
 
+	double pipeline_timing_map[PIPELINE_MEASURE_MAX]; /**< Continuous measure of pipeline timing. */
+	unsigned pipeline_timing_i; /**< Index of the measure. */
+	double pipeline_timing_max; /**< Maximum time used to pipeline. */
+
+	double update_timing_map[PIPELINE_MEASURE_MAX]; /**< Continuous measure of update timing. */
+	unsigned update_timing_i; /**< Index of the measure. */
+	double update_timing_min; /**< Minimum time used to update. */
+
 	const adv_crtc* crtc_selected; /**< Current crtc, pointer in the crtc_vector. */
 	adv_crtc crtc_effective; /**< Current modified crtc. */
 	const adv_crtc* crtc_map[VIDEO_CRTC_MAX];
@@ -923,6 +942,7 @@ adv_error advance_video_mode_init(struct advance_video_context* context, struct
 void advance_video_mode_done(struct advance_video_context* context);
 adv_error advance_video_mode_update(struct advance_video_context* context);
 
+void advance_video_thread_wait(struct advance_video_context* context);
 void advance_video_reconfigure(struct advance_video_context* context, struct advance_video_config_context* config);
 
 void advance_video_skip(struct advance_video_context* context, struct advance_estimate_context* estimate_context, struct advance_record_context* record_context);
@@ -930,6 +950,8 @@ void advance_video_sync(struct advance_video_context* context, struct advance_so
 void advance_video_frame(struct advance_video_context* context, struct advance_record_context* record_context, struct advance_ui_context* ui_context, const struct osd_bitmap* game, const struct osd_bitmap* debug, const osd_rgb_t* debug_palette, unsigned debug_palette_size, adv_bool skip_flag);
 void advance_sound_frame(struct advance_sound_context* context, struct advance_record_context* record_context, struct advance_video_context* video_context, struct advance_safequit_context* safequit_context, const short* sample_buffer, unsigned sample_count, unsigned sample_recount, adv_bool normal_speed);
 
+adv_bool advance_video_is_programmable(const struct advance_video_context* context);
+
 /***************************************************************************/
 /* Global */
 
@@ -951,6 +973,7 @@ unsigned lang_identify_text(int lang, const char* text);
 struct advance_global_config_context {
 	int difficulty; /**< Difficulty level. DIFFICULTY_NONE for default. */
 	int lang; /**< Language. -1 for default. */
+	adv_bool autosave; /**< If config should be autosaved. */
 	adv_bool freeplay_flag; /**< Free play switch. */
 	adv_bool quiet_flag; /**< Be quiet on message printing. */
 	adv_bool mutedemo_flag; /**< Mute demo mode. */
@@ -1040,3 +1063,4 @@ static inline double advance_timer(void)
 }
 
 #endif
+
