@@ -11,7 +11,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details. 
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -107,29 +107,29 @@ static struct language LANG_ASIA =
 { 0, 0, { "Asia", "Japan", "Japanese" } };
 
 static struct language LANG[] = {
-{ &LANG_USA, "usa", { "USA", "US", "America", "American" } },
-{ &LANG_USA, "canada", { "Canada" } },
-{ &LANG_EUROPE, "england", { "England", "British" } },
-{ &LANG_EUROPE, "italy", { "Italy", "Italian" } },
-{ &LANG_EUROPE, "germany", { "Germany", "German" } },
-{ &LANG_EUROPE, "spain", { "Spain", "Hispanic", "Spanish" } },
-{ &LANG_EUROPE, "austria", { "Austria" } },
-{ &LANG_EUROPE, "norway", { "Norway", "Norwegian" } },
-{ &LANG_EUROPE, "france", { "France", "French" } },
-{ &LANG_EUROPE, "denmark", { "Denmark" } },
-{ &LANG_ASIA, "japan", { "Japan", "Japanese" } },
-{ &LANG_ASIA, "korea", { "Korea" } },
-{ &LANG_ASIA, "china", { "China" } },
-{ &LANG_ASIA, "hongkong", { "Hong Kong", "Hong-Kong" } },
-{ &LANG_ASIA, "taiwan", { "Taiwan" } },
-{ 0, 0 }
+	{ &LANG_USA, "usa", { "USA", "US", "America", "American" } },
+	{ &LANG_USA, "canada", { "Canada" } },
+	{ &LANG_EUROPE, "england", { "England", "British" } },
+	{ &LANG_EUROPE, "italy", { "Italy", "Italian" } },
+	{ &LANG_EUROPE, "germany", { "Germany", "German" } },
+	{ &LANG_EUROPE, "spain", { "Spain", "Hispanic", "Spanish" } },
+	{ &LANG_EUROPE, "austria", { "Austria" } },
+	{ &LANG_EUROPE, "norway", { "Norway", "Norwegian" } },
+	{ &LANG_EUROPE, "france", { "France", "French" } },
+	{ &LANG_EUROPE, "denmark", { "Denmark" } },
+	{ &LANG_ASIA, "japan", { "Japan", "Japanese" } },
+	{ &LANG_ASIA, "korea", { "Korea" } },
+	{ &LANG_ASIA, "china", { "China" } },
+	{ &LANG_ASIA, "hongkong", { "Hong Kong", "Hong-Kong" } },
+	{ &LANG_ASIA, "taiwan", { "Taiwan" } },
+	{ 0, 0 }
 };
 
 static adv_bool lang_match(const struct language* id, const char* text)
 {
 	unsigned i;
 
-	for(i=0;id->tag[i];++i) {
+	for (i = 0; id->tag[i]; ++i) {
 		const char* j = strstr(text, id->tag[i]);
 		if (j != 0) {
 			if (j == text || !isalpha(j[-1])) {
@@ -188,7 +188,7 @@ int osd_display_loading_rom_message(const char* name, rom_load_data* romdata)
 
 	if (!romdata && name) {
 		/* it's a message */
-		if (!context->config.quiet_flag || strstr(name, "ERROR")!=0) {
+		if (!context->config.quiet_flag || strstr(name, "ERROR") != 0) {
 			target_err("%s", name);
 		}
 	}
@@ -226,7 +226,7 @@ static void config_customize_language(struct advance_global_context* context, in
 	i = current;
 	while (i->type != IPT_END) {
 		if (i->type == IPT_DIPSWITCH_NAME
-			&& (strstr(i->name, "Language")!=0 || strstr(i->name, "Territory")!=0 || strstr(i->name, "Country")!=0)) {
+			&& (strstr(i->name, "Language") != 0 || strstr(i->name, "Territory") != 0 || strstr(i->name, "Country") != 0)) {
 			input_port_entry* j;
 			input_port_entry* best;
 			unsigned best_value;
@@ -251,7 +251,7 @@ static void config_customize_language(struct advance_global_context* context, in
 			/* search the best */
 			best = begin;
 			best_value = lang_identify_text(context->config.lang, best->name);
-			for(j=begin;j!=end;++j) {
+			for (j = begin; j != end; ++j) {
 				unsigned value = lang_identify_text(context->config.lang, j->name);
 				if (value > best_value) {
 					best_value = value;
@@ -311,7 +311,7 @@ static void config_customize_difficulty(struct advance_global_context* context,
 	i = current;
 	while (i->type != IPT_END) {
 		if (i->type == IPT_DIPSWITCH_NAME
-			&& strcmp(i->name, "Difficulty")==0) {
+			&& strcmp(i->name, "Difficulty") == 0) {
 
 			/* the value is stored in the NAME item */
 			value = i;
@@ -347,26 +347,26 @@ static void config_customize_difficulty(struct advance_global_context* context,
 
 	/* get the list of names */
 	switch (context->config.difficulty) {
-	case DIFFICULTY_NONE :
+	case DIFFICULTY_NONE:
 		/* nothing to do */
 		return;
-	case DIFFICULTY_EASIEST :
+	case DIFFICULTY_EASIEST:
 		names = NAME_EASIEST;
 		names_secondary = NAME_EASY;
 		break;
-	case DIFFICULTY_EASY :
+	case DIFFICULTY_EASY:
 		names = NAME_EASY;
 		names_secondary = 0;
 		break;
-	case DIFFICULTY_MEDIUM :
+	case DIFFICULTY_MEDIUM:
 		names = NAME_MEDIUM;
 		names_secondary = NAME_EASY;
 		break;
-	case DIFFICULTY_HARD :
+	case DIFFICULTY_HARD:
 		names = NAME_HARD;
 		names_secondary = 0;
 		break;
-	case DIFFICULTY_HARDEST :
+	case DIFFICULTY_HARDEST:
 		names = NAME_HARDEST;
 		names_secondary = NAME_HARD;
 		break;
@@ -375,9 +375,9 @@ static void config_customize_difficulty(struct advance_global_context* context,
 	/* search an exact match */
 	if (!level) {
 		unsigned j;
-		for(j=0;names[j] && !level;++j) {
-			for(i=begin;i!=end && !level;++i) {
-				if (strcmp(names[j], i->name)==0) {
+		for (j = 0; names[j] && !level; ++j) {
+			for (i = begin; i != end && !level; ++i) {
+				if (strcmp(names[j], i->name) == 0) {
 					level = i;
 					break;
 				}
@@ -388,9 +388,9 @@ static void config_customize_difficulty(struct advance_global_context* context,
 	/* search a secondary match */
 	if (!level && names_secondary) {
 		unsigned j;
-		for(j=0;names[j] && !level;++j) {
-			for(i=begin;i!=end && !level;++i) {
-				if (strcmp(names_secondary[j], i->name)==0) {
+		for (j = 0; names[j] && !level; ++j) {
+			for (i = begin; i != end && !level; ++i) {
+				if (strcmp(names_secondary[j], i->name) == 0) {
 					level = i;
 					break;
 				}
@@ -402,19 +402,19 @@ static void config_customize_difficulty(struct advance_global_context* context,
 	if (!level) {
 		unsigned n = end - begin;
 		switch (context->config.difficulty) {
-		case DIFFICULTY_EASIEST :
+		case DIFFICULTY_EASIEST:
 			level = begin;
 			break;
-		case DIFFICULTY_EASY :
+		case DIFFICULTY_EASY:
 			level = begin + n / 4;
-				break;
-		case DIFFICULTY_MEDIUM :
+			break;
+		case DIFFICULTY_MEDIUM:
 			level = begin + n * 2 / 4;
 			break;
-		case DIFFICULTY_HARD :
+		case DIFFICULTY_HARD:
 			level = begin + n * 3 / 4;
 			break;
-		case DIFFICULTY_HARDEST :
+		case DIFFICULTY_HARDEST:
 			level = end - 1;
 			break;
 		}
@@ -520,11 +520,11 @@ static void config_customize_mutedemo(struct advance_global_context* context, in
 			/* read the value */
 			while (i->type == IPT_DIPSWITCH_SETTING) {
 				if (match_name) {
-					if (!exact && strcmp(i->name, "Off")==0) {
+					if (!exact && strcmp(i->name, "Off") == 0) {
 						exact = i;
 					}
 				} else {
-					if (!exact && strcmp(i->name, "Demo Sounds Off")==0) {
+					if (!exact && strcmp(i->name, "Demo Sounds Off") == 0) {
 						exact = i;
 					}
 				}
@@ -641,7 +641,7 @@ static void config_customize_input(struct advance_global_context* context, adv_c
 	i = current;
 	while (i->type != IPT_END) {
 		unsigned n;
-		for(n=glue_port_seq_begin(i->type);n<glue_port_seq_end(i->type);++n) {
+		for (n = glue_port_seq_begin(i->type); n < glue_port_seq_end(i->type); ++n) {
 			struct mame_port* p;
 			p = mame_port_find(glue_port_convert(i->type, i->player, glue_port_seqtype(i->type, n), i->name));
 			if (p != 0) {
@@ -700,6 +700,10 @@ void osd_customize_inputport_list(input_port_default_entry* defaults)
 	seq_set_1(&i->defaultseq, KEYCODE_F1_REAL);
 	i->name = "Help";
 
+	i = config_portdef_find(defaults, IPT_UI_KEYBOARD);
+	seq_set_1(&i->defaultseq, KEYCODE_PLUS_PAD);
+	i->name = "Keyboard";
+
 	i = config_portdef_find(defaults, IPT_UI_RECORD_START);
 	seq_set_2(&i->defaultseq, KEYCODE_ENTER, KEYCODE_LCONTROL);
 	i->name = "Record Start";
@@ -745,7 +749,7 @@ void osd_config_load_default(input_port_default_entry* backup, input_port_defaul
 	i = list;
 	while (i->type != IPT_END) {
 		unsigned n;
-		for(n=glue_port_seq_begin(i->type);n<glue_port_seq_end(i->type);++n) {
+		for (n = glue_port_seq_begin(i->type); n < glue_port_seq_end(i->type); ++n) {
 			struct mame_port* p;
 			p = mame_port_find(glue_port_convert(i->type, i->player, glue_port_seqtype(i->type, n), i->name));
 			if (p != 0) {
@@ -845,15 +849,50 @@ static void config_save_seq_default(unsigned port, unsigned* seq, unsigned seq_m
 		snprintf(tag_buffer, sizeof(tag_buffer), "input_map[%s]", p->name);
 
 		if (!seq || seq[0] == DIGITAL_SPECIAL_AUTO) {
-			log_std(("global: customize port default %s\n", tag_buffer));
+			const char* value = 0;
+			int store = 1;
 
-			conf_remove(cfg_context, "", tag_buffer);
+			/* parse the stored config and check if it's the same */
+			if (conf_string_section_get(cfg_context, "", tag_buffer, &value) != 0) {
+				/* not found, no need to delete it */
+				store = 0;
+			}
+
+			if (!value)
+				value = "<unset>";
+
+			if (store) {
+				log_std(("global: customize port default %s '%s' -> '<unset>'\n", tag_buffer, value));
+
+				conf_remove(cfg_context, "", tag_buffer);
+			}
 		} else {
-			advance_input_print_digital(value_buffer, sizeof(value_buffer), seq, seq_max);
+			const char* value = 0;
+			int store = 1;
 
-			log_std(("global: customize port %s %s\n", tag_buffer, value_buffer));
+			/* parse the stored config and check if it's the same */
+			if (conf_string_section_get(cfg_context, "", tag_buffer, &value) == 0) {
+				char* d = strdup(value);
+				unsigned store_seq[INPUT_MAP_MAX];
+				if (advance_input_parse_digital(store_seq, INPUT_MAP_MAX, d) == 0) {
+					if (memcmp(store_seq, seq, sizeof(store_seq)) == 0) {
+						/* found and equal, no need to save it */
+						store = 0;
+					}
+				}
+				free(d);
+			}
 
-			conf_string_set(cfg_context, "", tag_buffer, value_buffer);
+			if (!value)
+				value = "<unset>";
+
+			if (store) {
+				advance_input_print_digital(value_buffer, sizeof(value_buffer), seq, seq_max);
+
+				log_std(("global: customize port %s '%s' -> '%s'\n", tag_buffer, value, value_buffer));
+
+				conf_string_set(cfg_context, "", tag_buffer, value_buffer);
+			}
 		}
 	} else {
 		log_debug(("WARNING:global: customization for unknown port %d not saved\n", port));
@@ -879,21 +918,57 @@ static void config_save_seq(unsigned port, unsigned* seq, unsigned seq_max)
 	if (p) {
 		char tag_buffer[64];
 		char value_buffer[512];
+		const char* section = mame_section_name(game, cfg_context);
 
 		log_std(("global: setup port %s\n", p->name));
 
 		snprintf(tag_buffer, sizeof(tag_buffer), "input_map[%s]", p->name);
 
 		if (!seq || seq[0] == DIGITAL_SPECIAL_AUTO) {
-			log_std(("global: customize port default %s/%s\n", mame_section_name(game, cfg_context), tag_buffer));
+			const char* value = 0;
+			int store = 1;
 
-			conf_remove(cfg_context, mame_section_name(game, cfg_context), tag_buffer);
+			/* parse the stored config and check if it's the same */
+			if (conf_string_section_get(cfg_context, section, tag_buffer, &value) != 0) {
+				/* not found, no need to delete it */
+				store = 0;
+			}
+
+			if (!value)
+				value = "<unset>";
+
+			if (store) {
+				log_std(("global: customize port default %s/%s '%s' -> '<unset>'\n", section, tag_buffer, value));
+
+				conf_remove(cfg_context, section, tag_buffer);
+			}
 		} else {
-			advance_input_print_digital(value_buffer, sizeof(value_buffer), seq, seq_max);
+			const char* value = 0;
+			int store = 1;
 
-			log_std(("global: customize port %s/%s %s\n", mame_section_name(game, cfg_context), tag_buffer, value_buffer));
+			/* parse the stored config and check if it's the same */
+			if (conf_string_section_get(cfg_context, section, tag_buffer, &value) == 0) {
+				char* d = strdup(value);
+				unsigned store_seq[INPUT_MAP_MAX];
+				if (advance_input_parse_digital(store_seq, INPUT_MAP_MAX, d) == 0) {
+					if (memcmp(store_seq, seq, sizeof(store_seq)) == 0) {
+						/* found and equal, no need to save it */
+						store = 0;
+					}
+				}
+				free(d);
+			}
 
-			conf_set_if_different(cfg_context, mame_section_name(game, cfg_context), tag_buffer, value_buffer);
+			if (!value)
+				value = "<unset>";
+
+			if (store) {
+				advance_input_print_digital(value_buffer, sizeof(value_buffer), seq, seq_max);
+
+				log_std(("global: customize port %s/%s '%s' -> '%s'\n", section, tag_buffer, value, value_buffer));
+
+				conf_set_if_different(cfg_context, section, tag_buffer, value_buffer);
+			}
 		}
 	} else {
 		log_debug(("WARNING:global: customization for unknown port %d not saved\n", port));
@@ -975,12 +1050,12 @@ static void config_save_switchport(input_port_entry* def, input_port_entry* curr
 	log_debug(("global: config_save_switchport()\n"));
 
 	switch (current->type) {
-	case IPT_DIPSWITCH_NAME :
+	case IPT_DIPSWITCH_NAME:
 		tag = "input_dipswitch";
 		type = IPT_DIPSWITCH_SETTING;
 		break;
 #ifdef MESS
-	case IPT_CONFIG_NAME :
+	case IPT_CONFIG_NAME:
 		tag = "input_configswitch";
 		type = IPT_CONFIG_SETTING;
 		break;
@@ -990,7 +1065,7 @@ static void config_save_switchport(input_port_entry* def, input_port_entry* curr
 		return;
 	}
 
-	if (strcmp(current->name, DEF_STR(Unused))==0 || strcmp(current->name, DEF_STR(Unknown))==0) {
+	if (strcmp(current->name, DEF_STR(Unused)) == 0 || strcmp(current->name, DEF_STR(Unknown)) == 0) {
 		log_std(("WARNING:global: ignoring named Unknown/Unused switchport %d\n", current->type));
 		return;
 	}
@@ -1069,7 +1144,7 @@ void osd_config_save_default(input_port_default_entry* backup, input_port_defaul
 		if (i->name != 0) {
 			switch (type) {
 			default:
-				for(k=glue_port_seq_begin(type);k!=glue_port_seq_end(type);++k) {
+				for (k = glue_port_seq_begin(type); k != glue_port_seq_end(type); ++k) {
 					config_save_seqport_default(j, i, glue_port_seqtype(type, k));
 				}
 				break;
@@ -1095,15 +1170,15 @@ void osd_config_save(input_port_entry* backup, input_port_entry* list)
 
 		if (input_port_active(i)) {
 			switch (type) {
-			case IPT_DIPSWITCH_NAME :
-			case IPT_CONFIG_NAME :
+			case IPT_DIPSWITCH_NAME:
+			case IPT_CONFIG_NAME:
 				config_save_switchport(j, i);
 				break;
 			default:
 				if (analog) {
 					config_save_analogport(j, i);
 				}
-				for(k=glue_port_seq_begin(type);k!=glue_port_seq_end(type);++k) {
+				for (k = glue_port_seq_begin(type); k != glue_port_seq_end(type); ++k) {
 					config_save_seqport(j, i, glue_port_seqtype(type, k));
 				}
 				break;
@@ -1119,12 +1194,17 @@ void osd_config_save(input_port_entry* backup, input_port_entry* list)
 /* Initialization */
 
 static adv_conf_enum_int OPTION_DIFFICULTY[] = {
-{ "none", DIFFICULTY_NONE },
-{ "easiest", DIFFICULTY_EASIEST },
-{ "easy", DIFFICULTY_EASY },
-{ "normal", DIFFICULTY_MEDIUM },
-{ "hard", DIFFICULTY_HARD },
-{ "hardest", DIFFICULTY_HARDEST }
+	{ "none", DIFFICULTY_NONE },
+	{ "easiest", DIFFICULTY_EASIEST },
+	{ "easy", DIFFICULTY_EASY },
+	{ "normal", DIFFICULTY_MEDIUM },
+	{ "hard", DIFFICULTY_HARD },
+	{ "hardest", DIFFICULTY_HARDEST }
+};
+
+static adv_conf_enum_int OPTION_RESTORE[] = {
+	{ "save_at_exit", 1 },
+	{ "restore_at_exit", 0 },
 };
 
 #define OPTION_LANG_MAX 64
@@ -1134,17 +1214,18 @@ adv_error advance_global_init(struct advance_global_context* context, adv_conf*
 {
 	unsigned i;
 
+	conf_int_register_enum_default(cfg_context, "config", conf_enum(OPTION_RESTORE), 1);
 	conf_bool_register_default(cfg_context, "misc_quiet", 0);
 	conf_int_register_enum_default(cfg_context, "misc_difficulty", conf_enum(OPTION_DIFFICULTY), DIFFICULTY_NONE);
 
 	OPTION_LANG[0].value = "none";
 	OPTION_LANG[0].map = -1;
-	for(i=0;LANG[i].name && i+1<OPTION_LANG_MAX;++i) {
-		OPTION_LANG[i+1].value = LANG[i].name;
-		OPTION_LANG[i+1].map = i;
+	for (i = 0; LANG[i].name && i + 1 < OPTION_LANG_MAX; ++i) {
+		OPTION_LANG[i + 1].value = LANG[i].name;
+		OPTION_LANG[i + 1].map = i;
 	}
 
-	conf_int_register_enum_default(cfg_context, "misc_lang", OPTION_LANG, i+1, -1);
+	conf_int_register_enum_default(cfg_context, "misc_lang", OPTION_LANG, i + 1, -1);
 	conf_bool_register_default(cfg_context, "misc_freeplay", 0);
 	conf_bool_register_default(cfg_context, "misc_mutedemo", 0);
 	conf_float_register_limit_default(cfg_context, "display_pausebrightness", 0.0, 1.0, 1.0);
@@ -1162,6 +1243,7 @@ adv_error advance_global_config_load(struct advance_global_context* context, adv
 {
 	const char* s;
 
+	context->config.autosave = conf_int_get_default(cfg_context, "config");
 	context->config.quiet_flag = conf_bool_get_default(cfg_context, "misc_quiet");
 	context->config.difficulty = conf_int_get_default(cfg_context, "misc_difficulty");
 
