@@ -11,7 +11,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details. 
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -60,7 +60,7 @@ static int video_mode_menu(struct advance_video_context* context, struct advance
 	auto_index = advance_ui_menu_text_insert(&menu, "Auto");
 
 	crtc = 0;
-	for(i=0;i<context->state.crtc_mac;++i) {
+	for (i = 0; i < context->state.crtc_mac; ++i) {
 		char buffer[128];
 		mode_desc_print(context, buffer, sizeof(buffer), context->state.crtc_map[i]);
 		if (advance_ui_menu_text_insert(&menu, buffer) == selected)
@@ -90,7 +90,7 @@ static int video_mode_menu(struct advance_video_context* context, struct advance
 			advance_video_reconfigure(context, &config);
 
 			/* show at screen the new configuration name */
-			advance_ui_message(ui_context, mode_current_name(context));
+			advance_ui_message(ui_context, "%s", mode_current_name(context));
 		} else {
 			struct advance_video_config_context config = context->config;
 
@@ -128,7 +128,7 @@ static int video_pipeline_menu(struct advance_video_context* context, struct adv
 	snprintf(buffer, sizeof(buffer), "Video Pipeline (%d)", context->state.blit_pipeline_index);
 	advance_ui_menu_title_insert(&menu, buffer);
 
-	for(i=1,stage=video_pipeline_begin(&context->state.blit_pipeline[context->state.blit_pipeline_index]);stage!=video_pipeline_end(&context->state.blit_pipeline[context->state.blit_pipeline_index]);++stage,++i) {
+	for (i = 1, stage = video_pipeline_begin(&context->state.blit_pipeline[context->state.blit_pipeline_index]); stage != video_pipeline_end(&context->state.blit_pipeline[context->state.blit_pipeline_index]); ++stage, ++i) {
 		if (stage == video_pipeline_pivot(&context->state.blit_pipeline[context->state.blit_pipeline_index])) {
 			snprintf(buffer, sizeof(buffer), "(%d) %s", i, pipe_name(video_pipeline_vert(&context->state.blit_pipeline[context->state.blit_pipeline_index])->type));
 			advance_ui_menu_text_insert(&menu, buffer);
@@ -151,15 +151,27 @@ static int video_pipeline_menu(struct advance_video_context* context, struct adv
 	if (context->state.pipeline_measure_flag) {
 		advance_ui_menu_text_insert(&menu, "Time measure not completed");
 	} else {
-		for(i=0;i<PIPELINE_BLIT_MAX;++i) {
+		double timing = adv_measure_median(0.00001, 0.5, context->state.update_timing_map, PIPELINE_MEASURE_MAX);
+		snprintf(buffer, sizeof(buffer), "Last render %.2f (ms)", timing * 1000);
+		advance_ui_menu_text_insert(&menu, buffer);
+
+		timing = adv_measure_median(0.00001, 0.5, context->state.pipeline_timing_map, PIPELINE_MEASURE_MAX);
+		snprintf(buffer, sizeof(buffer), "Last write %.2f (ms)", timing * 1000);
+		advance_ui_menu_text_insert(&menu, buffer);
+
+		for (i = 0; i < PIPELINE_BLIT_MAX; ++i) {
 			const char* desc;
+			const char* select;
+			if (context->state.blit_pipeline_index == i)
+				select = "-> ";
+			else
+				select = "";
 			switch (i) {
-			case 0 : desc = "Buffer"; break;
-			case 1 : desc = "Direct"; break;
-			case 2 : desc = "Splitted"; break;
-			default : desc = "Unknown"; break;
+			case 0: desc = "Buffer"; break;
+			case 1: desc = "Direct"; break;
+			default: desc = "Unknown"; break;
 			}
-			snprintf(buffer, sizeof(buffer), "(%d) %s write %.2f (ms)", i, desc, context->state.pipeline_measure_result[i] * 1000);
+			snprintf(buffer, sizeof(buffer), "%s%s write %.2f (ms)", select, desc, context->state.pipeline_measure_result[i] * 1000);
 			advance_ui_menu_text_insert(&menu, buffer);
 		}
 	}
@@ -214,9 +226,13 @@ int osd2_video_menu(int selected, unsigned input)
 	int crash_index;
 	int exit_index;
 	char buffer[128];
+	char freq[32];
 	const char* text = 0;
 	const char* option = 0;
 
+	/* the menu data is not duplicated for the thread, so we have to wait for its completion */
+	advance_video_thread_wait(video_context);
+
 	if (selected >= 1)
 		selected = selected - 1;
 	else
@@ -225,12 +241,12 @@ int osd2_video_menu(int selected, unsigned input)
 	if (video_context->state.menu_sub_flag) {
 		int ret = 0;
 		switch (video_context->state.menu_sub_flag) {
-		case 1 : ret = video_mode_menu(video_context, ui_context, video_context->state.menu_sub_selected, input); break;
-		case 2 : ret = video_pipeline_menu(video_context, ui_context, video_context->state.menu_sub_selected, input); break;
+		case 1: ret = video_mode_menu(video_context, ui_context, video_context->state.menu_sub_selected, input); break;
+		case 2: ret = video_pipeline_menu(video_context, ui_context, video_context->state.menu_sub_selected, input); break;
 		}
 		switch (ret) {
-		case -1 : return -1; /* hide interface */
-		case 0 : video_context->state.menu_sub_flag = 0; video_context->state.menu_sub_selected = 1; break; /* close submenu */
+		case -1: return -1;  /* hide interface */
+		case 0: video_context->state.menu_sub_flag = 0; video_context->state.menu_sub_selected = 1; break;  /* close submenu */
 		default: video_context->state.menu_sub_selected = ret; break;
 		}
 		return selected + 1;
@@ -253,21 +269,21 @@ int osd2_video_menu(int selected, unsigned input)
 	resolution_index = advance_ui_menu_option_insert(&menu, "Mode...", video_context->config.resolution_buffer);
 
 	if (!video_context->state.game_vector_flag) {
-		if (strcmp(video_context->config.resolution_buffer, "auto")==0) {
+		if (strcmp(video_context->config.resolution_buffer, "auto") == 0) {
 			switch (mode_current_magnify(video_context)) {
-			default :
-			case 1 : text = "Magnify [1]"; break;
-			case 2 : text = "Magnify [2]"; break;
-			case 3 : text = "Magnify [3]"; break;
-			case 4 : text = "Magnify [4]"; break;
+			default:
+			case 1: text = "Magnify [1]"; break;
+			case 2: text = "Magnify [2]"; break;
+			case 3: text = "Magnify [3]"; break;
+			case 4: text = "Magnify [4]"; break;
 			}
 			switch (video_context->config.magnify_factor) {
-			default :
-			case 0 : option = "auto"; break;
-			case 1 : option = "1"; break;
-			case 2 : option = "2"; break;
-			case 3 : option = "3"; break;
-			case 4 : option = "4"; break;
+			default:
+			case 0: option = "auto"; break;
+			case 1: option = "1"; break;
+			case 2: option = "2"; break;
+			case 3: option = "3"; break;
+			case 4: option = "4"; break;
 			}
 			magnify_index = advance_ui_menu_option_insert(&menu, text, option);
 		} else {
@@ -277,16 +293,16 @@ int osd2_video_menu(int selected, unsigned input)
 		advance_ui_menu_title_insert(&menu, "Options");
 
 		switch (mode_current_stretch(video_context)) {
-		case STRETCH_NONE : text = "Resize [no]"; break;
-		case STRETCH_INTEGER_XY : text = "Resize [integer]"; break;
-		case STRETCH_INTEGER_X_FRACTIONAL_Y : text = "Resize [mixed]"; break;
-		case STRETCH_FRACTIONAL_XY : text = "Resize [fractional]"; break;
+		case STRETCH_NONE: text = "Resize [no]"; break;
+		case STRETCH_INTEGER_XY: text = "Resize [integer]"; break;
+		case STRETCH_INTEGER_X_FRACTIONAL_Y: text = "Resize [mixed]"; break;
+		case STRETCH_FRACTIONAL_XY: text = "Resize [fractional]"; break;
 		}
 		switch (video_context->config.stretch) {
-		case STRETCH_NONE : option = "no"; break;
-		case STRETCH_INTEGER_XY : option = "integer"; break;
-		case STRETCH_INTEGER_X_FRACTIONAL_Y : option = "mixed"; break;
-		case STRETCH_FRACTIONAL_XY : option = "fractional"; break;
+		case STRETCH_NONE: option = "no"; break;
+		case STRETCH_INTEGER_XY: option = "integer"; break;
+		case STRETCH_INTEGER_X_FRACTIONAL_Y: option = "mixed"; break;
+		case STRETCH_FRACTIONAL_XY: option = "fractional"; break;
 		}
 		stretch_index = advance_ui_menu_option_insert(&menu, text, option);
 	} else {
@@ -298,22 +314,22 @@ int osd2_video_menu(int selected, unsigned input)
 	}
 
 	switch (video_index()) {
-	case MODE_FLAGS_INDEX_PALETTE8 :
+	case MODE_FLAGS_INDEX_PALETTE8:
 		text = "Color [palette8]";
 		break;
-	case MODE_FLAGS_INDEX_BGR8 :
+	case MODE_FLAGS_INDEX_BGR8:
 		text = "Color [bgr8]";
 		break;
-	case MODE_FLAGS_INDEX_BGR15 :
+	case MODE_FLAGS_INDEX_BGR15:
 		text = "Color [bgr15]";
 		break;
-	case MODE_FLAGS_INDEX_BGR16 :
+	case MODE_FLAGS_INDEX_BGR16:
 		text = "Color [bgr16]";
 		break;
-	case MODE_FLAGS_INDEX_BGR32 :
+	case MODE_FLAGS_INDEX_BGR32:
 		text = "Color [bgr32]";
 		break;
-	case MODE_FLAGS_INDEX_YUY2 :
+	case MODE_FLAGS_INDEX_YUY2:
 		text = "Color [yuy2]";
 		break;
 	default:
@@ -321,25 +337,25 @@ int osd2_video_menu(int selected, unsigned input)
 		break;
 	}
 	switch (video_context->config.index) {
-	case MODE_FLAGS_INDEX_NONE :
+	case MODE_FLAGS_INDEX_NONE:
 		option = "auto";
-			break;
-	case MODE_FLAGS_INDEX_PALETTE8 :
+		break;
+	case MODE_FLAGS_INDEX_PALETTE8:
 		option = "palette8";
 		break;
-	case MODE_FLAGS_INDEX_BGR8 :
+	case MODE_FLAGS_INDEX_BGR8:
 		option = "bgr8";
 		break;
-	case MODE_FLAGS_INDEX_BGR15 :
+	case MODE_FLAGS_INDEX_BGR15:
 		option = "bgr15";
 		break;
-	case MODE_FLAGS_INDEX_BGR16 :
+	case MODE_FLAGS_INDEX_BGR16:
 		option = "bgr16";
 		break;
-	case MODE_FLAGS_INDEX_BGR32 :
+	case MODE_FLAGS_INDEX_BGR32:
 		option = "bgr32";
 		break;
-	case MODE_FLAGS_INDEX_YUY2 :
+	case MODE_FLAGS_INDEX_YUY2:
 		option = "yuy2";
 		break;
 	default:
@@ -349,72 +365,77 @@ int osd2_video_menu(int selected, unsigned input)
 	index_index = advance_ui_menu_option_insert(&menu, text, option);
 
 	switch (video_context->state.combine) {
-	case COMBINE_NONE : text = "Resize Effect [no]"; break;
-	case COMBINE_MAXMIN : text = "Resize Effect [max]"; break;
-	case COMBINE_MEAN : text = "Resize Effect [mean]"; break;
-	case COMBINE_FILTER : text = "Resize Effect [filter]"; break;
-	case COMBINE_SCALE : text = "Resize Effect [scale]"; break;
-	case COMBINE_LQ : text = "Resize Effect [lq]"; break;
-	case COMBINE_HQ : text = "Resize Effect [hq]"; break;
+	case COMBINE_NONE: text = "Resize Effect [no]"; break;
+	case COMBINE_MAXMIN: text = "Resize Effect [max]"; break;
+	case COMBINE_MEAN: text = "Resize Effect [mean]"; break;
+	case COMBINE_FILTER: text = "Resize Effect [filter]"; break;
+	case COMBINE_SCALEX: text = "Resize Effect [scalex]"; break;
+	case COMBINE_SCALEK: text = "Resize Effect [scalek]"; break;
+	case COMBINE_HQ: text = "Resize Effect [hq]"; break;
+	case COMBINE_XBR: text = "Resize Effect [xbr]"; break;
 	}
 	switch (video_context->config.combine) {
-	case COMBINE_AUTO : option = "auto"; break;
-	case COMBINE_NONE : option = "no"; break;
-	case COMBINE_MAXMIN : option = "max"; break;
-	case COMBINE_MEAN : option = "mean"; break;
-	case COMBINE_FILTER : option = "filter"; break;
-	case COMBINE_SCALE : option = "scale"; break;
-	case COMBINE_LQ : option = "lq"; break;
-	case COMBINE_HQ : option = "hq"; break;
+	case COMBINE_AUTO: option = "auto"; break;
+	case COMBINE_NONE: option = "no"; break;
+	case COMBINE_MAXMIN: option = "max"; break;
+	case COMBINE_MEAN: option = "mean"; break;
+	case COMBINE_FILTER: option = "filter"; break;
+	case COMBINE_SCALEX: option = "scalex"; break;
+	case COMBINE_SCALEK: option = "scalek"; break;
+	case COMBINE_HQ: option = "hq"; break;
+	case COMBINE_XBR: option = "xbr"; break;
 	}
 	combine_index = advance_ui_menu_option_insert(&menu, text, option);
 
 	switch (video_context->state.rgb_effect) {
-	case EFFECT_NONE : text = "Rgb Effect [no]"; break;
-	case EFFECT_RGB_TRIAD3PIX : text = "Rgb Effect [triad3dot]"; break;
-	case EFFECT_RGB_TRIAD6PIX : text = "Rgb Effect [triad6dot]"; break;
-	case EFFECT_RGB_TRIAD16PIX : text = "Rgb Effect [triad16dot]"; break;
-	case EFFECT_RGB_TRIADSTRONG3PIX : text = "Rgb Effect [triadstrong3dot]"; break;
-	case EFFECT_RGB_TRIADSTRONG6PIX : text = "Rgb Effect [triadstrong6dot]"; break;
-	case EFFECT_RGB_TRIADSTRONG16PIX : text = "Rgb Effect [triadstrong16dot]"; break;
-	case EFFECT_RGB_SCANDOUBLEHORZ : text = "Rgb Effect [scan2horz]"; break;
-	case EFFECT_RGB_SCANTRIPLEHORZ : text = "Rgb Effect [scan3horz]"; break;
-	case EFFECT_RGB_SCANDOUBLEVERT : text = "Rgb Effect [scan2vert]"; break;
-	case EFFECT_RGB_SCANTRIPLEVERT : text = "Rgb Effect [scan3vert]"; break;
+	case EFFECT_NONE: text = "Rgb Effect [no]"; break;
+	case EFFECT_RGB_TRIAD3PIX: text = "Rgb Effect [triad3dot]"; break;
+	case EFFECT_RGB_TRIAD6PIX: text = "Rgb Effect [triad6dot]"; break;
+	case EFFECT_RGB_TRIAD16PIX: text = "Rgb Effect [triad16dot]"; break;
+	case EFFECT_RGB_TRIADSTRONG3PIX: text = "Rgb Effect [triadstrong3dot]"; break;
+	case EFFECT_RGB_TRIADSTRONG6PIX: text = "Rgb Effect [triadstrong6dot]"; break;
+	case EFFECT_RGB_TRIADSTRONG16PIX: text = "Rgb Effect [triadstrong16dot]"; break;
+	case EFFECT_RGB_SCANDOUBLEHORZ: text = "Rgb Effect [scan2horz]"; break;
+	case EFFECT_RGB_SCANTRIPLEHORZ: text = "Rgb Effect [scan3horz]"; break;
+	case EFFECT_RGB_SCANDOUBLEVERT: text = "Rgb Effect [scan2vert]"; break;
+	case EFFECT_RGB_SCANTRIPLEVERT: text = "Rgb Effect [scan3vert]"; break;
 	}
 	switch (video_context->config.rgb_effect) {
-	case EFFECT_NONE : option = "no"; break;
-	case EFFECT_RGB_TRIAD3PIX : option = "triad3dot"; break;
-	case EFFECT_RGB_TRIAD6PIX : option = "triad6dot"; break;
-	case EFFECT_RGB_TRIAD16PIX : option = "triad16dot"; break;
-	case EFFECT_RGB_TRIADSTRONG3PIX : option = "triadstrong3dot"; break;
-	case EFFECT_RGB_TRIADSTRONG6PIX : option = "triadstrong6dot"; break;
-	case EFFECT_RGB_TRIADSTRONG16PIX : option = "triadstrong16dot"; break;
-	case EFFECT_RGB_SCANDOUBLEHORZ : option = "scan2horz"; break;
-	case EFFECT_RGB_SCANTRIPLEHORZ : option = "scan3horz"; break;
-	case EFFECT_RGB_SCANDOUBLEVERT : option = "scan2vert"; break;
-	case EFFECT_RGB_SCANTRIPLEVERT : option = "scan3vert"; break;
+	case EFFECT_NONE: option = "no"; break;
+	case EFFECT_RGB_TRIAD3PIX: option = "triad3dot"; break;
+	case EFFECT_RGB_TRIAD6PIX: option = "triad6dot"; break;
+	case EFFECT_RGB_TRIAD16PIX: option = "triad16dot"; break;
+	case EFFECT_RGB_TRIADSTRONG3PIX: option = "triadstrong3dot"; break;
+	case EFFECT_RGB_TRIADSTRONG6PIX: option = "triadstrong6dot"; break;
+	case EFFECT_RGB_TRIADSTRONG16PIX: option = "triadstrong16dot"; break;
+	case EFFECT_RGB_SCANDOUBLEHORZ: option = "scan2horz"; break;
+	case EFFECT_RGB_SCANTRIPLEHORZ: option = "scan3horz"; break;
+	case EFFECT_RGB_SCANDOUBLEVERT: option = "scan2vert"; break;
+	case EFFECT_RGB_SCANTRIPLEVERT: option = "scan3vert"; break;
 	}
 	effect_index = advance_ui_menu_option_insert(&menu, text, option);
 
 	switch (video_context->state.vsync_flag) {
-	case 0 : text = "Vsync [no]"; break;
-	case 1 : text = "Vsync [yes]"; break;
+	case 0: text = "Vsync [no]"; break;
+	case 1:
+		snprintf(freq, sizeof(freq), "Vsync [%.1f]", video_measured_vclock());
+		text = freq;
+		break;
 	}
 	switch (video_context->config.vsync_flag) {
-	case 0 : option = "no"; break;
-	case 1 : option = "yes"; break;
+	case 0: option = "no"; break;
+	case 1: option = "yes"; break;
 	}
 	vsync_index = advance_ui_menu_option_insert(&menu, text, option);
 
 #ifdef USE_SMP
 	switch (video_context->config.smp_flag) {
-	case 0 : text = "SMP [no]"; break;
-	case 1 : text = "SMP [yes]"; break;
+	case 0: text = "SMP [no]"; break;
+	case 1: text = "SMP [yes]"; break;
 	}
 	switch (video_context->config.smp_flag) {
-	case 0 : option = "no"; break;
-	case 1 : option = "yes"; break;
+	case 0: option = "no"; break;
+	case 1: option = "yes"; break;
 	}
 	smp_index = advance_ui_menu_option_insert(&menu, text, option);
 #else
@@ -483,44 +504,45 @@ int osd2_video_menu(int selected, unsigned input)
 		if (selected == combine_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.combine) {
-			case COMBINE_AUTO : config.combine = COMBINE_NONE; break;
-			case COMBINE_NONE : config.combine = COMBINE_MAXMIN; break;
-			case COMBINE_MAXMIN : config.combine = COMBINE_MEAN; break;
-			case COMBINE_MEAN : config.combine = COMBINE_FILTER; break;
-			case COMBINE_FILTER : config.combine = COMBINE_SCALE; break;
-			case COMBINE_SCALE : config.combine = COMBINE_LQ; break;
-			case COMBINE_LQ : config.combine = COMBINE_HQ; break;
-			case COMBINE_HQ : config.combine = COMBINE_AUTO; break;
+			case COMBINE_AUTO: config.combine = COMBINE_NONE; break;
+			case COMBINE_NONE: config.combine = COMBINE_MAXMIN; break;
+			case COMBINE_MAXMIN: config.combine = COMBINE_MEAN; break;
+			case COMBINE_MEAN: config.combine = COMBINE_FILTER; break;
+			case COMBINE_FILTER: config.combine = COMBINE_SCALEX; break;
+			case COMBINE_SCALEX: config.combine = COMBINE_SCALEK; break;
+			case COMBINE_SCALEK: config.combine = COMBINE_HQ; break;
+			case COMBINE_HQ: config.combine = COMBINE_XBR; break;
+			case COMBINE_XBR: config.combine = COMBINE_AUTO; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == effect_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.rgb_effect) {
-			case EFFECT_NONE : config.rgb_effect = EFFECT_RGB_TRIAD3PIX; break;
-			case EFFECT_RGB_TRIAD3PIX : config.rgb_effect = EFFECT_RGB_TRIADSTRONG3PIX; break;
-			case EFFECT_RGB_TRIADSTRONG3PIX : config.rgb_effect = EFFECT_RGB_TRIAD6PIX; break;
-			case EFFECT_RGB_TRIAD6PIX : config.rgb_effect = EFFECT_RGB_TRIADSTRONG6PIX; break;
-			case EFFECT_RGB_TRIADSTRONG6PIX : config.rgb_effect = EFFECT_RGB_TRIAD16PIX; break;
-			case EFFECT_RGB_TRIAD16PIX : config.rgb_effect = EFFECT_RGB_TRIADSTRONG16PIX; break;
-			case EFFECT_RGB_TRIADSTRONG16PIX : config.rgb_effect = EFFECT_RGB_SCANDOUBLEHORZ; break;
-			case EFFECT_RGB_SCANDOUBLEHORZ : config.rgb_effect = EFFECT_RGB_SCANTRIPLEHORZ; break;
-			case EFFECT_RGB_SCANTRIPLEHORZ : config.rgb_effect = EFFECT_RGB_SCANDOUBLEVERT; break;
-			case EFFECT_RGB_SCANDOUBLEVERT : config.rgb_effect = EFFECT_RGB_SCANTRIPLEVERT; break;
-			case EFFECT_RGB_SCANTRIPLEVERT : config.rgb_effect = EFFECT_NONE; break;
+			case EFFECT_NONE: config.rgb_effect = EFFECT_RGB_TRIAD3PIX; break;
+			case EFFECT_RGB_TRIAD3PIX: config.rgb_effect = EFFECT_RGB_TRIADSTRONG3PIX; break;
+			case EFFECT_RGB_TRIADSTRONG3PIX: config.rgb_effect = EFFECT_RGB_TRIAD6PIX; break;
+			case EFFECT_RGB_TRIAD6PIX: config.rgb_effect = EFFECT_RGB_TRIADSTRONG6PIX; break;
+			case EFFECT_RGB_TRIADSTRONG6PIX: config.rgb_effect = EFFECT_RGB_TRIAD16PIX; break;
+			case EFFECT_RGB_TRIAD16PIX: config.rgb_effect = EFFECT_RGB_TRIADSTRONG16PIX; break;
+			case EFFECT_RGB_TRIADSTRONG16PIX: config.rgb_effect = EFFECT_RGB_SCANDOUBLEHORZ; break;
+			case EFFECT_RGB_SCANDOUBLEHORZ: config.rgb_effect = EFFECT_RGB_SCANTRIPLEHORZ; break;
+			case EFFECT_RGB_SCANTRIPLEHORZ: config.rgb_effect = EFFECT_RGB_SCANDOUBLEVERT; break;
+			case EFFECT_RGB_SCANDOUBLEVERT: config.rgb_effect = EFFECT_RGB_SCANTRIPLEVERT; break;
+			case EFFECT_RGB_SCANTRIPLEVERT: config.rgb_effect = EFFECT_NONE; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == vsync_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.vsync_flag) {
-			case 0 : config.vsync_flag = 1; break;
-			case 1 : config.vsync_flag = 0; break;
+			case 0: config.vsync_flag = 1; break;
+			case 1: config.vsync_flag = 0; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == smp_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.smp_flag) {
-			case 0 : config.smp_flag = 1; break;
-			case 1 : config.smp_flag = 0; break;
+			case 0: config.smp_flag = 1; break;
+			case 1: config.smp_flag = 0; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == magnify_index) {
@@ -533,22 +555,22 @@ int osd2_video_menu(int selected, unsigned input)
 		} else if (selected == index_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.index) {
-			case MODE_FLAGS_INDEX_NONE : config.index = MODE_FLAGS_INDEX_PALETTE8; break;
-			case MODE_FLAGS_INDEX_PALETTE8 : config.index = MODE_FLAGS_INDEX_BGR8; break;
-			case MODE_FLAGS_INDEX_BGR8 : config.index = MODE_FLAGS_INDEX_BGR15; break;
-			case MODE_FLAGS_INDEX_BGR15 : config.index = MODE_FLAGS_INDEX_BGR16; break;
-			case MODE_FLAGS_INDEX_BGR16 : config.index = MODE_FLAGS_INDEX_BGR32; break;
-			case MODE_FLAGS_INDEX_BGR32 : config.index = MODE_FLAGS_INDEX_YUY2; break;
-			case MODE_FLAGS_INDEX_YUY2 : config.index = MODE_FLAGS_INDEX_NONE; break;
+			case MODE_FLAGS_INDEX_NONE: config.index = MODE_FLAGS_INDEX_PALETTE8; break;
+			case MODE_FLAGS_INDEX_PALETTE8: config.index = MODE_FLAGS_INDEX_BGR8; break;
+			case MODE_FLAGS_INDEX_BGR8: config.index = MODE_FLAGS_INDEX_BGR15; break;
+			case MODE_FLAGS_INDEX_BGR15: config.index = MODE_FLAGS_INDEX_BGR16; break;
+			case MODE_FLAGS_INDEX_BGR16: config.index = MODE_FLAGS_INDEX_BGR32; break;
+			case MODE_FLAGS_INDEX_BGR32: config.index = MODE_FLAGS_INDEX_YUY2; break;
+			case MODE_FLAGS_INDEX_YUY2: config.index = MODE_FLAGS_INDEX_NONE; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == stretch_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.stretch) {
-			case STRETCH_NONE : config.stretch = STRETCH_FRACTIONAL_XY; break;
-			case STRETCH_INTEGER_XY : config.stretch = STRETCH_NONE; break;
-			case STRETCH_INTEGER_X_FRACTIONAL_Y : config.stretch = STRETCH_INTEGER_XY; break;
-			case STRETCH_FRACTIONAL_XY : config.stretch = STRETCH_INTEGER_X_FRACTIONAL_Y; break;
+			case STRETCH_NONE: config.stretch = STRETCH_FRACTIONAL_XY; break;
+			case STRETCH_INTEGER_XY: config.stretch = STRETCH_NONE; break;
+			case STRETCH_INTEGER_X_FRACTIONAL_Y: config.stretch = STRETCH_INTEGER_XY; break;
+			case STRETCH_FRACTIONAL_XY: config.stretch = STRETCH_INTEGER_X_FRACTIONAL_Y; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		}
@@ -558,44 +580,45 @@ int osd2_video_menu(int selected, unsigned input)
 		if (selected == combine_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.combine) {
-			case COMBINE_AUTO : config.combine = COMBINE_HQ; break;
-			case COMBINE_NONE : config.combine = COMBINE_AUTO; break;
-			case COMBINE_MAXMIN : config.combine = COMBINE_NONE; break;
-			case COMBINE_MEAN : config.combine = COMBINE_MAXMIN; break;
-			case COMBINE_FILTER : config.combine = COMBINE_MEAN; break;
-			case COMBINE_SCALE : config.combine = COMBINE_FILTER; break;
-			case COMBINE_LQ : config.combine = COMBINE_SCALE; break;
-			case COMBINE_HQ : config.combine = COMBINE_LQ; break;
+			case COMBINE_AUTO: config.combine = COMBINE_XBR; break;
+			case COMBINE_NONE: config.combine = COMBINE_AUTO; break;
+			case COMBINE_MAXMIN: config.combine = COMBINE_NONE; break;
+			case COMBINE_MEAN: config.combine = COMBINE_MAXMIN; break;
+			case COMBINE_FILTER: config.combine = COMBINE_MEAN; break;
+			case COMBINE_SCALEX: config.combine = COMBINE_FILTER; break;
+			case COMBINE_SCALEK: config.combine = COMBINE_SCALEX; break;
+			case COMBINE_HQ: config.combine = COMBINE_SCALEK; break;
+			case COMBINE_XBR: config.combine = COMBINE_HQ; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == effect_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.rgb_effect) {
-			case EFFECT_NONE : config.rgb_effect = EFFECT_RGB_SCANTRIPLEVERT; break;
-			case EFFECT_RGB_TRIAD3PIX : config.rgb_effect = EFFECT_NONE; break;
-			case EFFECT_RGB_TRIADSTRONG3PIX : config.rgb_effect = EFFECT_RGB_TRIAD3PIX; break;
-			case EFFECT_RGB_TRIAD6PIX : config.rgb_effect = EFFECT_RGB_TRIADSTRONG3PIX; break;
-			case EFFECT_RGB_TRIADSTRONG6PIX : config.rgb_effect = EFFECT_RGB_TRIAD6PIX; break;
-			case EFFECT_RGB_TRIAD16PIX : config.rgb_effect = EFFECT_RGB_TRIADSTRONG6PIX; break;
-			case EFFECT_RGB_TRIADSTRONG16PIX : config.rgb_effect = EFFECT_RGB_TRIAD16PIX; break;
-			case EFFECT_RGB_SCANDOUBLEHORZ : config.rgb_effect = EFFECT_RGB_TRIADSTRONG16PIX; break;
-			case EFFECT_RGB_SCANTRIPLEHORZ : config.rgb_effect = EFFECT_RGB_SCANDOUBLEHORZ; break;
-			case EFFECT_RGB_SCANDOUBLEVERT : config.rgb_effect = EFFECT_RGB_SCANTRIPLEHORZ; break;
-			case EFFECT_RGB_SCANTRIPLEVERT : config.rgb_effect = EFFECT_RGB_SCANDOUBLEVERT; break;
+			case EFFECT_NONE: config.rgb_effect = EFFECT_RGB_SCANTRIPLEVERT; break;
+			case EFFECT_RGB_TRIAD3PIX: config.rgb_effect = EFFECT_NONE; break;
+			case EFFECT_RGB_TRIADSTRONG3PIX: config.rgb_effect = EFFECT_RGB_TRIAD3PIX; break;
+			case EFFECT_RGB_TRIAD6PIX: config.rgb_effect = EFFECT_RGB_TRIADSTRONG3PIX; break;
+			case EFFECT_RGB_TRIADSTRONG6PIX: config.rgb_effect = EFFECT_RGB_TRIAD6PIX; break;
+			case EFFECT_RGB_TRIAD16PIX: config.rgb_effect = EFFECT_RGB_TRIADSTRONG6PIX; break;
+			case EFFECT_RGB_TRIADSTRONG16PIX: config.rgb_effect = EFFECT_RGB_TRIAD16PIX; break;
+			case EFFECT_RGB_SCANDOUBLEHORZ: config.rgb_effect = EFFECT_RGB_TRIADSTRONG16PIX; break;
+			case EFFECT_RGB_SCANTRIPLEHORZ: config.rgb_effect = EFFECT_RGB_SCANDOUBLEHORZ; break;
+			case EFFECT_RGB_SCANDOUBLEVERT: config.rgb_effect = EFFECT_RGB_SCANTRIPLEHORZ; break;
+			case EFFECT_RGB_SCANTRIPLEVERT: config.rgb_effect = EFFECT_RGB_SCANDOUBLEVERT; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == vsync_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.vsync_flag) {
-			case 0 : config.vsync_flag = 1; break;
-			case 1 : config.vsync_flag = 0; break;
+			case 0: config.vsync_flag = 1; break;
+			case 1: config.vsync_flag = 0; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == smp_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.smp_flag) {
-			case 0 : config.smp_flag = 1; break;
-			case 1 : config.smp_flag = 0;  break;
+			case 0: config.smp_flag = 1; break;
+			case 1: config.smp_flag = 0;  break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == magnify_index) {
@@ -608,22 +631,22 @@ int osd2_video_menu(int selected, unsigned input)
 		} else if (selected == index_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.index) {
-			case MODE_FLAGS_INDEX_NONE : config.index = MODE_FLAGS_INDEX_YUY2; break;
-			case MODE_FLAGS_INDEX_PALETTE8 : config.index = MODE_FLAGS_INDEX_NONE; break;
-			case MODE_FLAGS_INDEX_BGR8 : config.index = MODE_FLAGS_INDEX_PALETTE8; break;
-			case MODE_FLAGS_INDEX_BGR15 : config.index = MODE_FLAGS_INDEX_BGR8; break;
-			case MODE_FLAGS_INDEX_BGR16 : config.index = MODE_FLAGS_INDEX_BGR15; break;
-			case MODE_FLAGS_INDEX_BGR32 : config.index = MODE_FLAGS_INDEX_BGR16; break;
-			case MODE_FLAGS_INDEX_YUY2 : config.index = MODE_FLAGS_INDEX_BGR32; break;
+			case MODE_FLAGS_INDEX_NONE: config.index = MODE_FLAGS_INDEX_YUY2; break;
+			case MODE_FLAGS_INDEX_PALETTE8: config.index = MODE_FLAGS_INDEX_NONE; break;
+			case MODE_FLAGS_INDEX_BGR8: config.index = MODE_FLAGS_INDEX_PALETTE8; break;
+			case MODE_FLAGS_INDEX_BGR15: config.index = MODE_FLAGS_INDEX_BGR8; break;
+			case MODE_FLAGS_INDEX_BGR16: config.index = MODE_FLAGS_INDEX_BGR15; break;
+			case MODE_FLAGS_INDEX_BGR32: config.index = MODE_FLAGS_INDEX_BGR16; break;
+			case MODE_FLAGS_INDEX_YUY2: config.index = MODE_FLAGS_INDEX_BGR32; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		} else if (selected == stretch_index) {
 			struct advance_video_config_context config = video_context->config;
 			switch (config.stretch) {
-			case STRETCH_NONE : config.stretch = STRETCH_INTEGER_XY; break;
-			case STRETCH_INTEGER_XY : config.stretch = STRETCH_INTEGER_X_FRACTIONAL_Y; break;
-			case STRETCH_INTEGER_X_FRACTIONAL_Y : config.stretch = STRETCH_FRACTIONAL_XY; break;
-			case STRETCH_FRACTIONAL_XY : config.stretch = STRETCH_NONE; break;
+			case STRETCH_NONE: config.stretch = STRETCH_INTEGER_XY; break;
+			case STRETCH_INTEGER_XY: config.stretch = STRETCH_INTEGER_X_FRACTIONAL_Y; break;
+			case STRETCH_INTEGER_X_FRACTIONAL_Y: config.stretch = STRETCH_FRACTIONAL_XY; break;
+			case STRETCH_FRACTIONAL_XY: config.stretch = STRETCH_NONE; break;
 			}
 			advance_video_reconfigure(video_context, &config);
 		}
@@ -681,15 +704,15 @@ static int audio_pipeline_menu(struct advance_sound_context* sound_context, stru
 		const char* to;
 
 		switch (sound_context->state.input_mode) {
-		case SOUND_MODE_MONO : from = "mono"; break;
-		case SOUND_MODE_STEREO : from = "stereo"; break;
-		case SOUND_MODE_SURROUND : from = "surround"; break;
+		case SOUND_MODE_MONO: from = "mono"; break;
+		case SOUND_MODE_STEREO: from = "stereo"; break;
+		case SOUND_MODE_SURROUND: from = "surround"; break;
 		default: from = "unknown"; break;
 		}
 		switch (sound_context->state.output_mode) {
-		case SOUND_MODE_MONO : to = "mono"; break;
-		case SOUND_MODE_STEREO : to = "stereo"; break;
-		case SOUND_MODE_SURROUND : to = "surround"; break;
+		case SOUND_MODE_MONO: to = "mono"; break;
+		case SOUND_MODE_STEREO: to = "stereo"; break;
+		case SOUND_MODE_SURROUND: to = "surround"; break;
 		default: to = "unknown"; break;
 		}
 
@@ -766,6 +789,9 @@ int osd2_audio_menu(int selected, unsigned input)
 	const char* text = 0;
 	const char* option = 0;
 
+	/* the menu data is not duplicated for the thread, so we have to wait for its completion */
+	advance_video_thread_wait(video_context);
+
 	if (selected >= 1)
 		selected = selected - 1;
 	else
@@ -774,11 +800,11 @@ int osd2_audio_menu(int selected, unsigned input)
 	if (sound_context->state.menu_sub_flag) {
 		int ret = 0;
 		switch (sound_context->state.menu_sub_flag) {
-		case 1 : ret = audio_pipeline_menu(sound_context, ui_context, sound_context->state.menu_sub_selected, input); break;
+		case 1: ret = audio_pipeline_menu(sound_context, ui_context, sound_context->state.menu_sub_selected, input); break;
 		}
 		switch (ret) {
-		case -1 : return -1; /* hide interface */
-		case 0 : sound_context->state.menu_sub_flag = 0; sound_context->state.menu_sub_selected = 1; break; /* close submenu */
+		case -1: return -1;  /* hide interface */
+		case 0: sound_context->state.menu_sub_flag = 0; sound_context->state.menu_sub_selected = 1; break;  /* close submenu */
 		default: sound_context->state.menu_sub_selected = ret; break;
 		}
 		return selected + 1;
@@ -789,15 +815,15 @@ int osd2_audio_menu(int selected, unsigned input)
 	advance_ui_menu_title_insert(&menu, "Audio Menu");
 
 	switch (sound_context->state.output_mode) {
-	case SOUND_MODE_MONO : text = "Mode [mono]"; break;
-	case SOUND_MODE_STEREO : text = "Mode [stereo]"; break;
-	case SOUND_MODE_SURROUND : text = "Mode [surround]"; break;
+	case SOUND_MODE_MONO: text = "Mode [mono]"; break;
+	case SOUND_MODE_STEREO: text = "Mode [stereo]"; break;
+	case SOUND_MODE_SURROUND: text = "Mode [surround]"; break;
 	}
 	switch (sound_context->config.mode) {
-	case SOUND_MODE_AUTO : option = "auto"; break;
-	case SOUND_MODE_MONO : option = "mono"; break;
-	case SOUND_MODE_STEREO : option = "stereo"; break;
-	case SOUND_MODE_SURROUND : option = "surround"; break;
+	case SOUND_MODE_AUTO: option = "auto"; break;
+	case SOUND_MODE_MONO: option = "mono"; break;
+	case SOUND_MODE_STEREO: option = "stereo"; break;
+	case SOUND_MODE_SURROUND: option = "surround"; break;
 	}
 	mode_index = advance_ui_menu_option_insert(&menu, text, option);
 
@@ -884,10 +910,10 @@ int osd2_audio_menu(int selected, unsigned input)
 		} else if (selected == mode_index) {
 			struct advance_sound_config_context config = sound_context->config;
 			switch (config.mode) {
-			case SOUND_MODE_AUTO : config.mode = SOUND_MODE_MONO; break;
-			case SOUND_MODE_MONO : config.mode = SOUND_MODE_STEREO; break;
-			case SOUND_MODE_STEREO : config.mode = SOUND_MODE_SURROUND; break;
-			case SOUND_MODE_SURROUND : config.mode = SOUND_MODE_AUTO; break;
+			case SOUND_MODE_AUTO: config.mode = SOUND_MODE_MONO; break;
+			case SOUND_MODE_MONO: config.mode = SOUND_MODE_STEREO; break;
+			case SOUND_MODE_STEREO: config.mode = SOUND_MODE_SURROUND; break;
+			case SOUND_MODE_SURROUND: config.mode = SOUND_MODE_AUTO; break;
 			}
 			advance_sound_reconfigure(sound_context, &config);
 		} else if (selected == eql_index) {
@@ -921,10 +947,10 @@ int osd2_audio_menu(int selected, unsigned input)
 		} else if (selected == mode_index) {
 			struct advance_sound_config_context config = sound_context->config;
 			switch (config.mode) {
-			case SOUND_MODE_AUTO : config.mode = SOUND_MODE_SURROUND; break;
-			case SOUND_MODE_MONO : config.mode = SOUND_MODE_AUTO; break;
-			case SOUND_MODE_STEREO : config.mode = SOUND_MODE_MONO; break;
-			case SOUND_MODE_SURROUND : config.mode = SOUND_MODE_STEREO; break;
+			case SOUND_MODE_AUTO: config.mode = SOUND_MODE_SURROUND; break;
+			case SOUND_MODE_MONO: config.mode = SOUND_MODE_AUTO; break;
+			case SOUND_MODE_STEREO: config.mode = SOUND_MODE_MONO; break;
+			case SOUND_MODE_SURROUND: config.mode = SOUND_MODE_STEREO; break;
 			}
 			advance_sound_reconfigure(sound_context, &config);
 		} else if (selected == eql_index) {
