@@ -11,7 +11,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details. 
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -36,7 +36,7 @@
 
 #include <math.h>
 
-/** Max framskip factor */
+/** Max frameskip factor */
 #define SYNC_MAX 4
 
 /**
@@ -50,14 +50,14 @@ void advance_video_update_skip(struct advance_video_context* context)
 	if (context->config.frameskip_factor >= 1.0) {
 		context->state.skip_level_full = SYNC_MAX;
 		context->state.skip_level_skip = 0;
-	} else if (context->config.frameskip_factor >= 1.0-1.0/SYNC_MAX) {
-		context->state.skip_level_full = SYNC_MAX -1;
+	} else if (context->config.frameskip_factor >= 1.0 - 1.0 / SYNC_MAX) {
+		context->state.skip_level_full = SYNC_MAX - 1;
 		context->state.skip_level_skip = 1;
-	} else if (context->config.frameskip_factor <= 1.0/SYNC_MAX) {
+	} else if (context->config.frameskip_factor <= 1.0 / SYNC_MAX) {
 		context->state.skip_level_full = 1;
-		context->state.skip_level_skip = SYNC_MAX -1;
+		context->state.skip_level_skip = SYNC_MAX - 1;
 	} else if (context->config.frameskip_factor >= 0.5) {
-		context->state.skip_level_full = context->config.frameskip_factor / (1-context->config.frameskip_factor);
+		context->state.skip_level_full = context->config.frameskip_factor / (1 - context->config.frameskip_factor);
 		if (context->state.skip_level_full < 1)
 			context->state.skip_level_full = 1;
 		if (context->state.skip_level_full >= SYNC_MAX)
@@ -65,7 +65,7 @@ void advance_video_update_skip(struct advance_video_context* context)
 		context->state.skip_level_skip = 1;
 	} else {
 		context->state.skip_level_full = 1;
-		context->state.skip_level_skip = (1-context->config.frameskip_factor) / context->config.frameskip_factor;
+		context->state.skip_level_skip = (1 - context->config.frameskip_factor) / context->config.frameskip_factor;
 		if (context->state.skip_level_skip < 1)
 			context->state.skip_level_skip = 1;
 		if (context->state.skip_level_skip >= SYNC_MAX)
@@ -74,7 +74,7 @@ void advance_video_update_skip(struct advance_video_context* context)
 }
 
 /**
- * Decremnt by one the frame skip state.
+ * Decrement by one the frame skip state.
  */
 adv_bool advance_video_skip_dec(struct advance_video_context* context)
 {
@@ -132,7 +132,7 @@ void advance_video_update_sync(struct advance_video_context* context)
 	context->state.sync_warming_up_flag = 1;
 
 	if (rate > 10 && rate < 300) {
-		context->state.mode_vclock = video_rate_scale_down(rate, reference);
+		context->state.mode_vclock = rate;
 		context->state.vsync_flag = context->config.vsync_flag;
 	} else {
 		/* out of range, surely NOT supported */
@@ -226,15 +226,15 @@ static void video_skip_recompute(struct advance_video_context* context, struct a
 
 	assert(skip <= full);
 
-	if (full <= step) {
+	if (full < step) {
 		/* full frame rate */
 		context->state.skip_level_full = SYNC_MAX;
 		context->state.skip_level_skip = 0;
 	} else if (skip >= step /* (this check is implicit on the next one) */
-		|| skip * SYNC_MAX + full >= step * (SYNC_MAX+1)
+		|| skip * SYNC_MAX + full >= step * (SYNC_MAX + 1)
 	) {
 		/* if the maximum skip plus one isn't enought use a special management */
-		/* (the plus one is to remove jumping on the compare border) */
+		/* (the plus one is to avoid to continously activate and deactivate skipping) */
 		if (context->state.turbo_flag || context->state.fastest_flag) {
 			/* null frame rate */
 			context->state.skip_level_full = 1;
@@ -317,9 +317,9 @@ static double video_frame_wait(double current, double expected)
 static void video_frame_sync(struct advance_video_context* context)
 {
 	double current;
-	double expected;
+	double begin;
 
-	current = advance_timer();
+	begin = current = advance_timer();
 
 	if (context->state.sync_warming_up_flag) {
 		/* syncronize the first time */
@@ -336,78 +336,85 @@ static void video_frame_sync(struct advance_video_context* context)
 
 		log_debug(("advance:sync: throttle warming up\n"));
 	} else {
-		double time_before_sync, time_after_delay, time_after_sync;
-
-		time_before_sync = current;
-
-		expected = context->state.sync_last + context->state.skip_step * (1 + context->state.sync_skip_counter);
-		context->state.sync_skip_counter = 0;
-
-		/* take only a part of the error, this increase the stability */
-		context->state.sync_pivot *= 0.99;
-
-		/* adjust with the previous error */
-		expected += context->state.sync_pivot;
-
 		/* the vsync is used only if all the frames are displayed */
 		if ((video_flags() & MODE_FLAGS_RETRACE_WAIT_SYNC) != 0
 			&& context->state.vsync_flag
 			&& context->state.skip_level_full == SYNC_MAX
 		) {
-			/* wait until the retrace is near (3% early), otherwise if the */
-			/* mode has a double freq the retrace may be the wrong one. */
-			double early = 0.03 / video_measured_vclock();
+			double limit;
+			double error;
+			double expected;
 
-			if (current < expected - early) {
-				current = video_frame_wait(current, expected - early);
-				time_after_delay = current;
+			expected = context->state.sync_last + 1.0 / context->state.mode_vclock;
+			context->state.sync_skip_counter = 0;
 
-				if (current < expected) {
-					double after;
-					video_wait_vsync();
-					after = advance_timer();
-					
-					if (after - current > 1.05 / (double)video_measured_vclock()) {
-						log_std(("ERROR:emu:video: sync wait too long. %g instead of %g (max %g)\n", after - current, early, 1.0 / (double)video_measured_vclock()));
-					} else if (after - current > 1.05 * early) {
-						log_std(("WARNING:emu:video: sync wait too long. %g instead of %g (max %g)\n", after - current, early, 1.0 / (double)video_measured_vclock()));
-					}
-					
-					/* if a sync complete correctly reset the error to 0 */
-					/* in this case the vsync is used for the correct clocking */
-					expected = after;
-					
-					current = after;
-				} else {
-					log_std(("ERROR:emu:video: sync delay too big\n"));
-				}
+			/*
+			 * Do not wait if we are too near at the vsync to not have false negative,
+			 * and to risk to wait for two of them.
+			 *
+			 * We don't wait the vsync if over the 94% of the frame.
+			 */
+			limit = 0.06 / context->state.mode_vclock;
+
+			if (current < expected - limit) {
+				video_wait_vsync();
+				current = advance_timer();
 			} else {
-				log_std(("ERROR:emu:video: too late for a video sync\n"));
 				current = video_frame_wait(current, expected);
-				time_after_delay = current;
 			}
+
+			error = expected - current;
+
+			if (error < -limit) {
+				log_std(("ERROR:advance:sync: wait %g too long. frame %g, reference %g (error %g)\n", current - begin, current - context->state.sync_last, 1.0 / context->state.mode_vclock, -error));
+			} else if (error > limit) {
+				log_std(("WARNING:advance:sync: wait %g too short. frame %g, reference %g (error %g)\n", current - begin, current - context->state.sync_last, 1.0 / context->state.mode_vclock, error));
+			}
+
+			/* save the time of the latest sync */
+			context->state.sync_last = current;
+
+			/* after a sync always reset the error to 0 */
+			context->state.sync_pivot = 0;
+		} else if ((video_flags() & (MODE_FLAGS_RETRACE_SCROLL_SYNC | MODE_FLAGS_RETRACE_WRITE_SYNC)) != 0
+			&& context->state.vsync_flag
+			&& context->state.skip_level_full == SYNC_MAX
+		) {
+			/*
+			 * We do nothing here, as we are going to vsync later when updating.
+			 */
+			context->state.sync_skip_counter = 0;
+
+			/* save the time of the latest sync */
+			context->state.sync_last = current;
+
+			/* after a sync always reset the error to 0 */
+			context->state.sync_pivot = 0;
 		} else {
+			double expected;
+
+			expected = context->state.sync_last + context->state.skip_step * (1 + context->state.sync_skip_counter);
+			context->state.sync_skip_counter = 0;
+
+			/* adjust with the previous error to try to recover it */
+			expected += context->state.sync_pivot;
+
 			current = video_frame_wait(current, expected);
-			time_after_delay = current;
+
+			/* save the time of the latest sync */
+			context->state.sync_last = current;
+
+			/* update the error state */
+			context->state.sync_pivot = expected - current;
+
+			if (context->state.sync_pivot < -context->state.skip_step * 16) {
+				/* if the error is too big (negative) the delay is unrecoverable */
+				/* generally it happen with a virtual terminal switch */
+				/* the best solution is to restart the sync computation */
+				advance_video_update_skip(context);
+				advance_video_update_sync(context);
+			}
 		}
-
-		time_after_sync = current;
-
-		/* update the error state */
-		context->state.sync_pivot = expected - current;
-
-		if (fabs(context->state.sync_pivot) > context->state.skip_step / 50)
-			log_std(("advance:sync: %.5f (err %6.1f%%) = %.5f + %.5f + %.5f < %.5f (compute + sleep + sync < max)\n", current - context->state.sync_last, context->state.sync_pivot * 100 / context->state.skip_step, time_before_sync - context->state.sync_last, time_after_delay - time_before_sync, time_after_sync - time_after_delay, context->state.skip_step));
-
-		if (context->state.sync_pivot < - context->state.skip_step * 16) {
-			/* if the error is too big (negative) the delay is unrecoverable */
-			/* generally it happen with a virtual terminal switch */
-			/* the best solution is to restart the sync computation */
-			advance_video_update_skip(context);
-			advance_video_update_sync(context);
-		}
-
-		context->state.sync_last = current;
 	}
 }
 
@@ -509,7 +516,7 @@ static void video_frame_skip_none(struct advance_video_context* context, struct
 	context->state.skip_flag = 0;
 	context->state.skip_level_counter = 0;
 
-	/* force a recoputation when needed */
+	/* force a recomputation when needed */
 	context->state.skip_warming_up_flag = 1;
 }
 
