@@ -11,7 +11,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details. 
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
@@ -47,6 +47,18 @@
 /***************************************************************************/
 /* Commands */
 
+static adv_bool video_is_normal_speed(struct advance_video_context* context)
+{
+	adv_bool normal_speed;
+
+	normal_speed = !context->state.turbo_flag
+		&& !context->state.fastest_flag
+		&& !context->state.measure_flag
+		&& context->state.sync_throttle_flag;
+
+	return normal_speed;
+}
+
 static void event_check(unsigned ordinal, adv_bool condition, int id, unsigned previous, unsigned* current)
 {
 	unsigned mask = 1 << ordinal;
@@ -63,7 +75,7 @@ static void event_check(unsigned ordinal, adv_bool condition, int id, unsigned p
 	}
 }
 
-static void video_command_event(struct advance_video_context* context, struct advance_safequit_context* safequit_context, int leds_status, adv_bool turbo_status, unsigned input)
+static void video_command_event(struct advance_video_context* context, struct advance_safequit_context* safequit_context, int leds_status, adv_bool turbo_status, unsigned input, int knocker_status)
 {
 	unsigned event_mask = 0;
 
@@ -94,6 +106,7 @@ static void video_command_event(struct advance_video_context* context, struct ad
 	event_check(24, advance_safequit_event_mask(safequit_context) & 0x2000, HARDWARE_SCRIPT_EVENT12, context->state.event_mask_old, &event_mask);
 	event_check(25, advance_safequit_event_mask(safequit_context) & 0x4000, HARDWARE_SCRIPT_EVENT13, context->state.event_mask_old, &event_mask);
 	event_check(26, advance_safequit_event_mask(safequit_context) & 0x8000, HARDWARE_SCRIPT_EVENT14, context->state.event_mask_old, &event_mask);
+	event_check(27, knocker_status != 0, HARDWARE_SCRIPT_KNOCKER, context->state.event_mask_old, &event_mask);
 
 	/* save the new status */
 	context->state.event_mask_old = event_mask;
@@ -109,30 +122,30 @@ static void video_command_resolution(struct advance_video_context* context, unsi
 
 	if ((input & OSD_INPUT_MODE_NEXT) != 0) {
 		show = 1;
-		if (strcmp(new_resolution, "auto")==0) {
+		if (strcmp(new_resolution, "auto") == 0) {
 			if (context->state.crtc_mac > 1) {
 				sncpy(new_resolution, MODE_NAME_MAX, crtc_name_get(context->state.crtc_map[1]));
 				modify = 1;
 			}
 		} else {
 			unsigned i;
-			for(i=0;i<context->state.crtc_mac;++i)
+			for (i = 0; i < context->state.crtc_mac; ++i)
 				if (context->state.crtc_map[i] == context->state.crtc_selected)
 					break;
-			if (i<context->state.crtc_mac && i+1<context->state.crtc_mac) {
-				sncpy(new_resolution, MODE_NAME_MAX, crtc_name_get(context->state.crtc_map[i+1]));
+			if (i < context->state.crtc_mac && i + 1 < context->state.crtc_mac) {
+				sncpy(new_resolution, MODE_NAME_MAX, crtc_name_get(context->state.crtc_map[i + 1]));
 				modify = 1;
 			}
 		}
 	} else if ((input & OSD_INPUT_MODE_PRED) != 0) {
 		show = 1;
-		if (strcmp(new_resolution, "auto")!=0) {
+		if (strcmp(new_resolution, "auto") != 0) {
 			unsigned i;
-			for(i=0;i<context->state.crtc_mac;++i)
+			for (i = 0; i < context->state.crtc_mac; ++i)
 				if (context->state.crtc_map[i] == context->state.crtc_selected)
 					break;
-			if (i<context->state.crtc_mac && i>0) {
-				sncpy(new_resolution, MODE_NAME_MAX, crtc_name_get(context->state.crtc_map[i-1]));
+			if (i < context->state.crtc_mac && i > 0) {
+				sncpy(new_resolution, MODE_NAME_MAX, crtc_name_get(context->state.crtc_map[i - 1]));
 				modify = 1;
 			}
 		}
@@ -188,13 +201,115 @@ static void video_command_pan(struct advance_video_context* context, unsigned in
 	}
 }
 
-static void video_command(struct advance_video_context* context, struct advance_estimate_context* estimate_context, struct advance_safequit_context* safequit_context, struct advance_ui_context* ui_context, adv_conf* cfg_context, int leds_status, unsigned input, adv_bool skip_flag)
+static void video_command_combine(struct advance_video_context* context, struct advance_ui_context* ui_context, adv_bool skip_flag)
+{
+	adv_bool decrease = 0;
+
+	if (advance_ui_buffer_active(ui_context) /* if no ui with buffer is active */
+		|| !video_is_normal_speed(context) /* if we are in normal runtime condition */
+	) {
+		/* reset the counter */
+		context->state.skip_level_combine_counter = 0;
+		context->state.skip_level_combine_total = 0;
+
+		/* don't adjust */
+		return;
+	}
+
+	/* increment the total counter */
+	++context->state.skip_level_combine_total;
+
+	/* periodically restart measuring */
+	if (context->state.skip_level_combine_total > 10000) {
+		/* restart counting */
+		context->state.skip_level_combine_counter = 0;
+		context->state.skip_level_combine_total = 0;
+	}
+
+	if (context->config.smp_flag /* we have a dedicated CPU for rendering */
+		&& context->state.skip_level_combine_total > 100 /* some frames have been rendered */
+	) {
+		double frame;
+		double update;
+		double pipeline;
+
+		if (context->state.vsync_flag) {
+			frame = 1.0 / context->state.mode_vclock;
+		} else {
+			frame = 1.0 / context->state.game_fps;
+		}
+
+		pipeline = adv_measure_mean(0.00001, 0.5, context->state.pipeline_timing_map, PIPELINE_MEASURE_MAX);
+		update = context->state.update_timing_min / TARGET_CLOCKS_PER_SEC;
+
+		if (pipeline + update > frame) {
+			log_std(("advance:skip: combine_time pipeline:%g + update:%g > %g\n", pipeline, update, frame));
+			decrease = 1;
+		}
+	}
+
+	if (!context->config.smp_flag /* we share the CPU with the game */
+		&& context->state.skip_level_skip != 0 /* if we are not 100% full speed */
+	) {
+		/* one more frame too slow */
+		++context->state.skip_level_combine_counter;
+
+		/* if we reached 3% of slow frames in 1000 frames */
+		if (
+			(context->state.skip_level_combine_total > 1000
+			&& context->state.skip_level_combine_counter > context->state.skip_level_combine_total / 33)
+			|| (context->state.skip_level_combine_total <= 1000
+			&& context->state.skip_level_combine_counter >= 30)
+		) {
+			log_std(("advance:skip: combine_skip_level %u %u\n", context->state.skip_level_combine_counter, context->state.skip_level_combine_total));
+			decrease = 1;
+		}
+	}
+
+	if (decrease) {
+		unsigned combine_max = context->config.combine_max;
+
+		/* use a simpler video video effect */
+		if (context->config.combine == COMBINE_AUTO) {
+			switch (combine_max) {
+			case COMBINE_SCALEX:
+				combine_max = COMBINE_NONE;
+				log_std(("advance:skip: decrease combine from scalex to none\n"));
+				break;
+			case COMBINE_SCALEK:
+				combine_max = COMBINE_SCALEX;
+				log_std(("advance:skip: decrease combine from scalek to scalex\n"));
+				break;
+			case COMBINE_XBR:
+				combine_max = COMBINE_SCALEK;
+				log_std(("advance:skip: decrease combine from xbr to scalek\n"));
+				break;
+			}
+		}
+
+		/* if something changed */
+		if (context->config.combine_max != combine_max) {
+			struct advance_video_config_context config = context->config;
+
+			config.combine_max = combine_max;
+
+			/* reconfigure */
+			advance_video_reconfigure(context, &config);
+
+			/* restart counting */
+			context->state.skip_level_combine_counter = 0;
+			context->state.skip_level_combine_total = 0;
+		}
+	}
+}
+
+static void video_command(struct advance_video_context* context, struct advance_estimate_context* estimate_context, struct advance_safequit_context* safequit_context, struct advance_ui_context* ui_context, adv_conf* cfg_context, int leds_status, unsigned input, adv_bool skip_flag, int knocker_status)
 {
 	/* increment the number of frames */
 	++context->state.frame_counter;
 
 	/* events */
-	video_command_event(context, safequit_context, leds_status, context->state.turbo_flag, input);
+	video_command_event(context, safequit_context, leds_status, context->state.turbo_flag, input, knocker_status);
 
 	/* scripts */
 	hardware_script_idle(SCRIPT_TIME_UNIT / context->state.game_fps);
@@ -290,7 +405,7 @@ static void video_command(struct advance_video_context* context, struct advance_
 			hardware_script_start(HARDWARE_SCRIPT_PLAY);
 		}
 	}
-	
+
 	if ((input & OSD_INPUT_TOGGLE_DEBUG) != 0) {
 		context->state.debugger_flag = !context->state.debugger_flag;
 		advance_video_invalidate_screen(context);
@@ -365,10 +480,10 @@ static void video_command(struct advance_video_context* context, struct advance_
 		/* 100% - 100% */
 		/* 10987654321 */
 		l = strlen(buffer);
-		if (l>=4 && isspace(buffer[l-4]))
-			buffer[l-4] = ADV_FONT_FIXSPACE;
-		if (l>=11 && isspace(buffer[l-11]))
-			buffer[l-11] = ADV_FONT_FIXSPACE;
+		if (l >= 4 && isspace(buffer[l - 4]))
+			buffer[l - 4] = ADV_FONT_FIXSPACE;
+		if (l >= 11 && isspace(buffer[l - 11]))
+			buffer[l - 11] = ADV_FONT_FIXSPACE;
 
 		advance_ui_direct_text(ui_context, buffer);
 
@@ -387,21 +502,42 @@ static void video_command(struct advance_video_context* context, struct advance_
 	video_command_resolution(context, input);
 
 	video_command_pan(context, input);
+
+	video_command_combine(context, ui_context, skip_flag);
 }
 
 /***************************************************************************/
 /* Frame */
 
-static adv_bool video_is_normal_speed(struct advance_video_context* context)
+static void video_frame_start(struct advance_video_context* context)
 {
-	adv_bool normal_speed;
+	update_start();
+}
 
-	normal_speed = !context->state.turbo_flag
-		&& !context->state.fastest_flag
-		&& !context->state.measure_flag
-		&& context->state.sync_throttle_flag;
+static void video_frame_stop(struct advance_video_context* context)
+{
+	adv_bool vsync = context->state.vsync_flag;
+	target_clock_t start;
+	target_clock_t stop;
 
-	return normal_speed;
+	start = target_clock();
+
+	/* if we already have synced on the wait */
+	if (vsync && (video_flags() & MODE_FLAGS_RETRACE_WAIT_SYNC) != 0)
+		vsync = 0;
+
+	update_stop(update_x_get(), update_y_get(), video_size_x(), video_size_y(), vsync);
+
+	stop = target_clock() - start;
+
+	context->state.update_timing_map[context->state.update_timing_i] = stop;
+	++context->state.update_timing_i;
+	if (context->state.update_timing_i == PIPELINE_MEASURE_MAX)
+		context->state.update_timing_i = 0;
+
+	/* keep track of the min time */
+	if (context->state.update_timing_min > stop)
+		context->state.update_timing_min = stop;
 }
 
 static void video_frame_update_now(struct advance_video_context* context, struct advance_sound_context* sound_context, struct advance_estimate_context* estimate_context, struct advance_record_context* record_context, struct advance_ui_context* ui_context, struct advance_safequit_context* safequit_context, const struct osd_bitmap* game, const struct osd_bitmap* debug, const osd_rgb_t* debug_palette, unsigned debug_palette_size, unsigned led, unsigned input, const short* sample_buffer, unsigned sample_count, unsigned sample_recount, adv_bool skip_flag)
@@ -415,6 +551,9 @@ static void video_frame_update_now(struct advance_video_context* context, struct
 	/* the frame syncronization is out of the time estimation */
 	advance_video_sync(context, sound_context, estimate_context, skip_flag);
 
+	/* start updating */
+	video_frame_start(context);
+
 	/* estimate the time */
 	advance_estimate_osd_begin(estimate_context);
 
@@ -426,12 +565,15 @@ static void video_frame_update_now(struct advance_video_context* context, struct
 
 	/* estimate the time */
 	advance_estimate_osd_end(estimate_context, skip_flag);
+
+	/* stop updating, this may include a vsync and it's out of the time estimation */
+	video_frame_stop(context);
 }
 
 /**
  * Wait the completion of the video thread.
  */
-static void video_thread_wait(struct advance_video_context* context)
+void advance_video_thread_wait(struct advance_video_context* context)
 {
 #ifdef USE_SMP
 	/* wait until the thread is ready */
@@ -529,7 +671,7 @@ static void video_frame_prepare(struct advance_video_context* context, struct ad
 		context->state.thread_state_skip_flag = skip_flag;
 
 		if (sample_count > context->state.thread_state_sample_max) {
-			log_std(("advance:thread: realloc sample buffer %d samples -> %d samples, %d bytes\n", context->state.thread_state_sample_max, 2*sample_count, sound_context->state.input_bytes_per_sample * 2 * sample_count));
+			log_std(("advance:thread: realloc sample buffer %d samples -> %d samples, %d bytes\n", context->state.thread_state_sample_max, 2 * sample_count, sound_context->state.input_bytes_per_sample * 2 * sample_count));
 			context->state.thread_state_sample_max = 2 * sample_count;
 			context->state.thread_state_sample_buffer = realloc(context->state.thread_state_sample_buffer, sound_context->state.input_bytes_per_sample * context->state.thread_state_sample_max);
 			assert(context->state.thread_state_sample_buffer);
@@ -555,7 +697,10 @@ static void video_frame_prepare(struct advance_video_context* context, struct ad
 static void video_frame_update(struct advance_video_context* context, struct advance_sound_context* sound_context, struct advance_estimate_context* estimate_context, struct advance_record_context* record_context, struct advance_ui_context* ui_context, struct advance_safequit_context* safequit_context, const struct osd_bitmap* game, const struct osd_bitmap* debug, const osd_rgb_t* debug_palette, unsigned debug_palette_size, unsigned led, unsigned input, const short* sample_buffer, unsigned sample_count, unsigned sample_recount, adv_bool skip_flag)
 {
 #ifdef USE_SMP
-	if (context->config.smp_flag && !context->state.debugger_flag) {
+	if (context->config.smp_flag && !context->state.debugger_flag
+	        /* we don't duplicate the UI data, and then we cannot access it directly from the thread */
+		&& !advance_ui_buffer_active(ui_context)
+	) {
 		pthread_mutex_lock(&context->state.thread_video_mutex);
 
 		log_debug(("advance:thread: signal\n"));
@@ -593,24 +738,27 @@ static void* video_thread(void* void_context)
 	pthread_mutex_lock(&context->state.thread_video_mutex);
 
 	while (1) {
+		adv_bool exit;
+
 		log_debug(("advance:thread: wait\n"));
 
-		/* wait for the start notification  */
+		/* wait for the start notification */
 		while (!context->state.thread_state_ready_flag && !context->state.thread_exit_flag) {
 			pthread_cond_wait(&context->state.thread_video_cond, &context->state.thread_video_mutex);
 		}
 
 		log_debug(("advance:thread: wakeup\n"));
 
+		exit = context->state.thread_exit_flag;
+
+		/* now we can start to draw outside the lock */
+		pthread_mutex_unlock(&context->state.thread_video_mutex);
+
 		/* check for exit */
-		if (context->state.thread_exit_flag) {
-			log_std(("advance:thread: stop\n"));
-			pthread_mutex_unlock(&context->state.thread_video_mutex);
+		if (exit) {
 			break;
 		}
 
-		pthread_mutex_unlock(&context->state.thread_video_mutex);
-
 		log_debug(("advance:thread: draw start\n"));
 
 		/* update the frame */
@@ -637,10 +785,10 @@ static void* video_thread(void* void_context)
 
 		pthread_mutex_lock(&context->state.thread_video_mutex);
 
-		/* notify that the data was used */
+		/* notify that the data was used, and a new one can be setup */
 		context->state.thread_state_ready_flag = 0;
 
-		/* signal the completion of the operation */
+		/* wakeup the main thread, signalign that the draw finished */
 		pthread_cond_signal(&context->state.thread_video_cond);
 	}
 
@@ -677,7 +825,8 @@ void advance_video_reconfigure(struct advance_video_context* context, struct adv
 	/* set the new config */
 	context->config = *config;
 
-	video_thread_wait(context);
+	/* we cannot change the video mode when the thread is running */
+	advance_video_thread_wait(context);
 
 	log_std(("emu:video: select mode %s\n", context->config.resolution_buffer));
 
@@ -703,6 +852,100 @@ void advance_video_reconfigure(struct advance_video_context* context, struct adv
 /***************************************************************************/
 /* OSD */
 
+void osd_stretch_palett16to32(void* dst, unsigned dst_dx, unsigned dst_dy, unsigned dst_dw, const void* src, unsigned src_dx, unsigned src_dy, int src_dw, const UINT32* palette)
+{
+	struct video_pipeline_struct pipeline;
+	struct advance_video_context* context = &CONTEXT.video;
+	adv_color_def def = adv_png_color_def(4);
+	unsigned combine = context->config.combine;
+
+	if (combine == COMBINE_AUTO)
+		combine = context->config.combine_max;
+
+	switch (combine) {
+	case COMBINE_MEAN:
+		combine = VIDEO_COMBINE_Y_MEAN | VIDEO_COMBINE_X_MEAN;
+		break;
+	case COMBINE_FILTER:
+		combine = VIDEO_COMBINE_Y_FILTER | VIDEO_COMBINE_X_FILTER;
+		break;
+	case COMBINE_SCALEX:
+		combine = VIDEO_COMBINE_Y_SCALEX;
+		break;
+	case COMBINE_SCALEK:
+		combine = VIDEO_COMBINE_Y_SCALEK;
+		break;
+#ifndef USE_BLIT_SMALL
+	case COMBINE_HQ:
+		combine = VIDEO_COMBINE_Y_HQ;
+		break;
+	case COMBINE_XBR:
+		combine = VIDEO_COMBINE_Y_XBR;
+		break;
+#endif
+	default:
+		combine = VIDEO_COMBINE_Y_NONE;
+		break;
+	}
+
+	video_pipeline_init(&pipeline);
+
+	video_pipeline_target(&pipeline, dst, dst_dw, video_color_def());
+
+	video_pipeline_palette16(&pipeline, dst_dx, dst_dy, src_dx, src_dy, src_dw, 2, 0, 0, palette, combine);
+
+	video_pipeline_blit(&pipeline, 0, 0, src);
+
+	video_pipeline_done(&pipeline);
+}
+
+void osd_stretch_32to32(void* dst, unsigned dst_dx, unsigned dst_dy, unsigned dst_dw, const void* src, unsigned src_dx, unsigned src_dy, int src_dw)
+{
+	struct video_pipeline_struct pipeline;
+	struct advance_video_context* context = &CONTEXT.video;
+	adv_color_def def = adv_png_color_def(4);
+	unsigned combine = context->config.combine;
+
+	if (combine == COMBINE_AUTO)
+		combine = context->config.combine_max;
+
+	switch (combine) {
+	case COMBINE_MEAN:
+		combine = VIDEO_COMBINE_Y_MEAN | VIDEO_COMBINE_X_MEAN;
+		break;
+	case COMBINE_FILTER:
+		combine = VIDEO_COMBINE_Y_FILTER | VIDEO_COMBINE_X_FILTER;
+		break;
+	case COMBINE_SCALEX:
+		combine = VIDEO_COMBINE_Y_SCALEX;
+		break;
+	case COMBINE_SCALEK:
+		combine = VIDEO_COMBINE_Y_SCALEK;
+		break;
+#ifndef USE_BLIT_SMALL
+	case COMBINE_HQ:
+		combine = VIDEO_COMBINE_Y_HQ;
+		break;
+	case COMBINE_XBR:
+		combine = VIDEO_COMBINE_Y_XBR;
+		break;
+#endif
+	default:
+		combine = VIDEO_COMBINE_Y_NONE;
+		break;
+	}
+
+	video_pipeline_init(&pipeline);
+
+	video_pipeline_target(&pipeline, dst, dst_dw, video_color_def());
+
+	video_pipeline_direct(&pipeline, dst_dx, dst_dy, src_dx, src_dy, src_dw, 4, def, combine);
+
+	video_pipeline_blit(&pipeline, 0, 0, src);
+
+	video_pipeline_done(&pipeline);
+}
+
 /**
  * Initialize and start the video thread.
  */
@@ -752,7 +995,7 @@ void osd2_thread_done(void)
 	struct advance_video_context* context = &CONTEXT.video;
 
 	log_std(("osd: osd2_thread_done\n"));
-	video_thread_wait(context);
+	advance_video_thread_wait(context);
 
 	log_std(("advance:thread: exit signal\n"));
 	pthread_mutex_lock(&context->state.thread_video_mutex);
@@ -835,9 +1078,12 @@ void osd2_video_done(void)
 	mouseb_disable();
 	keyb_disable();
 
+	/* wait for thread end, before resetting the video mode */
+	advance_video_thread_wait(context);
+
 	advance_video_mode_done(context);
 
-        /* print the speed measure */
+	/* print the speed measure */
 	if (context->state.measure_flag
 		&& context->state.measure_stop > context->state.measure_start) {
 		target_out("%g\n", (double)(context->state.measure_stop - context->state.measure_start) / TARGET_CLOCKS_PER_SEC);
@@ -918,7 +1164,7 @@ void osd2_palette(const osd_mask_t* mask, const osd_rgb_t* palette, unsigned siz
 	}
 
 	context->state.palette_dirty_flag = 1;
-	for(i=0;i<size;++i) {
+	for (i = 0; i < size; ++i) {
 		context->state.palette_map[i].red = osd_rgb_red(palette[i]);
 		context->state.palette_map[i].green = osd_rgb_green(palette[i]);
 		context->state.palette_map[i].blue = osd_rgb_blue(palette[i]);
@@ -926,12 +1172,12 @@ void osd2_palette(const osd_mask_t* mask, const osd_rgb_t* palette, unsigned siz
 
 	if (dirty_size > 0 && dirty_size == context->state.palette_dirty_total) {
 		/* the last element must be masked */
-		for(i=0;i<dirty_size - 1;++i) {
+		for (i = 0; i < dirty_size - 1; ++i) {
 			context->state.palette_dirty_map[i] |= mask[i];
 		}
 		context->state.palette_dirty_map[i] |= mask[i] & context->state.palette_dirty_mask;
 	} else {
-		for(i=0;i<dirty_size;++i) {
+		for (i = 0; i < dirty_size; ++i) {
 			context->state.palette_dirty_map[i] |= mask[i];
 		}
 	}
@@ -1013,7 +1259,7 @@ static int int_compare(const void* void_a, const void* void_b)
 /** Number of frames on which distribute the latency error. */
 #define AUDIOVIDEO_DISTRIBUTE_COUNT 4
 
-int osd2_frame(const struct osd_bitmap* game, const struct osd_bitmap* debug, const osd_rgb_t* debug_palette, unsigned debug_palette_size, unsigned led, unsigned input, const short* sample_buffer, unsigned sample_count)
+int osd2_frame(const struct osd_bitmap* game, const struct osd_bitmap* debug, const osd_rgb_t* debug_palette, unsigned debug_palette_size, unsigned led, unsigned input, const short* sample_buffer, unsigned sample_count, unsigned knocker)
 {
 	struct advance_video_context* context = &CONTEXT.video;
 
@@ -1052,13 +1298,13 @@ int osd2_frame(const struct osd_bitmap* game, const struct osd_bitmap* debug, co
 		int median_map[AUDIOVIDEO_MEASURE_MAX];
 
 		/* sort the last errors values */
-		for(i=0;i<AUDIOVIDEO_MEASURE_MAX;++i) {
+		for (i = 0; i < AUDIOVIDEO_MEASURE_MAX; ++i) {
 			median_map[i] = context->state.av_sync_map[i];
 		}
 		qsort(median_map, AUDIOVIDEO_MEASURE_MAX, sizeof(median_map[0]), int_compare);
 
 		/* get the median value from the most recent errors */
-		latency_median = median_map[AUDIOVIDEO_MEASURE_MAX/2];
+		latency_median = median_map[AUDIOVIDEO_MEASURE_MAX / 2];
 
 		/* if playing at normal speed */
 		if (latency_median >= -latency_limit && latency_median <= latency_limit) {
@@ -1073,7 +1319,7 @@ int osd2_frame(const struct osd_bitmap* game, const struct osd_bitmap* debug, co
 
 		if (latency_diff <= -AUDIOVIDEO_NEAR_STEP_COUNT || latency_diff >= AUDIOVIDEO_NEAR_STEP_COUNT) {
 			/* report the big error */
-			log_std(("WARNING:emu:video: audio/video syncronization correction of %d samples\n", latency_diff));
+			log_std(("WARNING:emu:video: audio/video syncronization correction of %d samples. Try increasing the 'sound_latency' option.\n", latency_diff));
 		} else if (latency_diff <= -3 || latency_diff >= 3) {
 			/* report the error, an error of 1,2 samples is the normal condition */
 			log_std(("emu:video: audio/video syncronization correction of %d samples\n", latency_diff));
@@ -1095,7 +1341,7 @@ int osd2_frame(const struct osd_bitmap* game, const struct osd_bitmap* debug, co
 		/* generally happen that the DMA buffer underflow, */
 		/* reporting a fill state instead of an empty one. */
 		if (sample_recount < sample_limit) {
-			log_std(("WARNING:emu:video: too small sound samples %d adjusted to %d\n", sample_recount, sample_limit));
+			log_std(("WARNING:emu:video: too few sound samples %d adjusted to %d\n", sample_recount, sample_limit));
 			sample_recount = sample_limit;
 		}
 
@@ -1117,7 +1363,7 @@ int osd2_frame(const struct osd_bitmap* game, const struct osd_bitmap* debug, co
 	}
 
 	/* update the global info */
-	video_command(&CONTEXT.video, &CONTEXT.estimate, &CONTEXT.safequit, &CONTEXT.ui, CONTEXT.cfg, led, input, skip_flag);
+	video_command(&CONTEXT.video, &CONTEXT.estimate, &CONTEXT.safequit, &CONTEXT.ui, CONTEXT.cfg, led, input, skip_flag, knocker);
 	advance_video_skip(&CONTEXT.video, &CONTEXT.estimate, &CONTEXT.record);
 	advance_input_update(&CONTEXT.input, &CONTEXT.safequit, CONTEXT.video.state.pause_flag);
 
@@ -1172,78 +1418,79 @@ static unsigned video_orientation_compute(unsigned orientation, adv_bool rol, ad
 /* Config */
 
 static adv_conf_enum_int OPTION_RESIZE[] = {
-{ "none", STRETCH_NONE },
-{ "integer", STRETCH_INTEGER_XY },
-{ "fractional", STRETCH_FRACTIONAL_XY },
-{ "mixed", STRETCH_INTEGER_X_FRACTIONAL_Y }
+	{ "none", STRETCH_NONE },
+	{ "integer", STRETCH_INTEGER_XY },
+	{ "fractional", STRETCH_FRACTIONAL_XY },
+	{ "mixed", STRETCH_INTEGER_X_FRACTIONAL_Y }
 };
 
 static adv_conf_enum_int OPTION_RESIZEEFFECT[] = {
-{ "auto", COMBINE_AUTO },
-{ "none", COMBINE_NONE },
-{ "max", COMBINE_MAXMIN },
-{ "mean", COMBINE_MEAN },
-{ "filter", COMBINE_FILTER },
-{ "scale", COMBINE_SCALE },
-{ "lq", COMBINE_LQ },
-{ "hq", COMBINE_HQ }
+	{ "auto", COMBINE_AUTO },
+	{ "none", COMBINE_NONE },
+	{ "max", COMBINE_MAXMIN },
+	{ "mean", COMBINE_MEAN },
+	{ "filter", COMBINE_FILTER },
+	{ "scalex", COMBINE_SCALEX },
+	{ "scalek", COMBINE_SCALEK },
+	{ "hq", COMBINE_HQ },
+	{ "xbr", COMBINE_XBR }
 };
 
 static adv_conf_enum_int OPTION_ADJUST[] = {
-{ "none", ADJUST_NONE },
-{ "x", ADJUST_ADJUST_X },
-{ "clock", ADJUST_ADJUST_CLOCK },
-{ "xclock", ADJUST_ADJUST_X | ADJUST_ADJUST_CLOCK },
-{ "generate_exact", ADJUST_GENERATE | ADJUST_ADJUST_X },
-{ "generate_y", ADJUST_GENERATE | ADJUST_ADJUST_X | ADJUST_ADJUST_Y },
-{ "generate_clock", ADJUST_GENERATE | ADJUST_ADJUST_X | ADJUST_ADJUST_CLOCK },
-{ "generate_yclock", ADJUST_GENERATE | ADJUST_ADJUST_X | ADJUST_ADJUST_Y | ADJUST_ADJUST_CLOCK | ADJUST_FAVORITE_SIZE_OVER_CLOCK },
-{ "generate_clocky", ADJUST_GENERATE | ADJUST_ADJUST_X | ADJUST_ADJUST_Y | ADJUST_ADJUST_CLOCK }
+	{ "none", ADJUST_NONE },
+	{ "x", ADJUST_ADJUST_X },
+	{ "clock", ADJUST_ADJUST_CLOCK },
+	{ "xclock", ADJUST_ADJUST_X | ADJUST_ADJUST_CLOCK },
+	{ "generate_exact", ADJUST_GENERATE | ADJUST_ADJUST_X },
+	{ "generate_y", ADJUST_GENERATE | ADJUST_ADJUST_X | ADJUST_ADJUST_Y },
+	{ "generate_clock", ADJUST_GENERATE | ADJUST_ADJUST_X | ADJUST_ADJUST_CLOCK },
+	{ "generate_yclock", ADJUST_GENERATE | ADJUST_ADJUST_X | ADJUST_ADJUST_Y | ADJUST_ADJUST_CLOCK | ADJUST_FAVORITE_SIZE_OVER_CLOCK },
+	{ "generate_clocky", ADJUST_GENERATE | ADJUST_ADJUST_X | ADJUST_ADJUST_Y | ADJUST_ADJUST_CLOCK }
 };
 
 static adv_conf_enum_int OPTION_MAGNIFY[] = {
-{ "auto", 0 },
-{ "1", 1 },
-{ "2", 2 },
-{ "3", 3 },
-{ "4", 4 }
+	{ "auto", 0 },
+	{ "1", 1 },
+	{ "2", 2 },
+	{ "3", 3 },
+	{ "4", 4 }
 };
 
 static adv_conf_enum_int OPTION_RESAMPLE[] = {
-{ "auto", -1 },
-{ "emulation", 0 },
-{ "internal", 1 }
+	{ "auto", -1 },
+	{ "emulation", 0 },
+	{ "internal", 1 }
 };
 
 static adv_conf_enum_int OPTION_RGBEFFECT[] = {
-{ "none", EFFECT_NONE },
-{ "triad3dot", EFFECT_RGB_TRIAD3PIX },
-{ "triad6dot", EFFECT_RGB_TRIAD6PIX },
-{ "triad16dot", EFFECT_RGB_TRIAD16PIX },
-{ "triadstrong3dot", EFFECT_RGB_TRIADSTRONG3PIX },
-{ "triadstrong6dot", EFFECT_RGB_TRIADSTRONG6PIX },
-{ "triadstrong16dot", EFFECT_RGB_TRIADSTRONG16PIX },
-{ "scan2horz", EFFECT_RGB_SCANDOUBLEHORZ },
-{ "scan2vert", EFFECT_RGB_SCANDOUBLEVERT },
-{ "scan3horz", EFFECT_RGB_SCANTRIPLEHORZ },
-{ "scan3vert", EFFECT_RGB_SCANTRIPLEVERT }
+	{ "none", EFFECT_NONE },
+	{ "triad3dot", EFFECT_RGB_TRIAD3PIX },
+	{ "triad6dot", EFFECT_RGB_TRIAD6PIX },
+	{ "triad16dot", EFFECT_RGB_TRIAD16PIX },
+	{ "triadstrong3dot", EFFECT_RGB_TRIADSTRONG3PIX },
+	{ "triadstrong6dot", EFFECT_RGB_TRIADSTRONG6PIX },
+	{ "triadstrong16dot", EFFECT_RGB_TRIADSTRONG16PIX },
+	{ "scan2horz", EFFECT_RGB_SCANDOUBLEHORZ },
+	{ "scan2vert", EFFECT_RGB_SCANDOUBLEVERT },
+	{ "scan3horz", EFFECT_RGB_SCANTRIPLEHORZ },
+	{ "scan3vert", EFFECT_RGB_SCANTRIPLEVERT }
 };
 
 static adv_conf_enum_int OPTION_INTERLACEEFFECT[] = {
-{ "none", EFFECT_NONE },
-{ "odd", EFFECT_INTERLACE_ODD },
-{ "even", EFFECT_INTERLACE_EVEN },
-{ "filter", EFFECT_INTERLACE_FILTER }
+	{ "none", EFFECT_NONE },
+	{ "odd", EFFECT_INTERLACE_ODD },
+	{ "even", EFFECT_INTERLACE_EVEN },
+	{ "filter", EFFECT_INTERLACE_FILTER }
 };
 
 static adv_conf_enum_int OPTION_INDEX[] = {
-{ "auto", MODE_FLAGS_INDEX_NONE },
-{ "palette8", MODE_FLAGS_INDEX_PALETTE8 },
-{ "bgr8", MODE_FLAGS_INDEX_BGR8 },
-{ "bgr15", MODE_FLAGS_INDEX_BGR15 },
-{ "bgr16", MODE_FLAGS_INDEX_BGR16 },
-{ "bgr32", MODE_FLAGS_INDEX_BGR32 },
-{ "yuy2", MODE_FLAGS_INDEX_YUY2 }
+	{ "auto", MODE_FLAGS_INDEX_NONE },
+	{ "palette8", MODE_FLAGS_INDEX_PALETTE8 },
+	{ "bgr8", MODE_FLAGS_INDEX_BGR8 },
+	{ "bgr15", MODE_FLAGS_INDEX_BGR15 },
+	{ "bgr16", MODE_FLAGS_INDEX_BGR16 },
+	{ "bgr32", MODE_FLAGS_INDEX_BGR32 },
+	{ "yuy2", MODE_FLAGS_INDEX_YUY2 }
 };
 
 adv_error advance_video_init(struct advance_video_context* context, adv_conf* cfg_context)
@@ -1252,7 +1499,7 @@ adv_error advance_video_init(struct advance_video_context* context, adv_conf* cf
 
 	/* other initialization */
 	context->state.av_sync_mac = 0;
-	for(i=0;i<AUDIOVIDEO_MEASURE_MAX;++i)
+	for (i = 0; i < AUDIOVIDEO_MEASURE_MAX; ++i)
 		context->state.av_sync_map[i] = 0;
 	context->state.latency_diff = 0;
 
@@ -1260,7 +1507,8 @@ adv_error advance_video_init(struct advance_video_context* context, adv_conf* cf
 	conf_bool_register_default(cfg_context, "display_vsync", 1);
 	conf_bool_register_default(cfg_context, "display_buffer", 0);
 	conf_int_register_enum_default(cfg_context, "display_resize", conf_enum(OPTION_RESIZE), STRETCH_FRACTIONAL_XY);
-	conf_int_register_enum_default(cfg_context, "display_magnify", conf_enum(OPTION_MAGNIFY), 1);
+	conf_int_register_enum_default(cfg_context, "display_magnify", conf_enum(OPTION_MAGNIFY), 0);
+	conf_int_register_default(cfg_context, "display_magnifysize", 640);
 	conf_int_register_enum_default(cfg_context, "display_adjust", conf_enum(OPTION_ADJUST), ADJUST_NONE);
 	conf_string_register_default(cfg_context, "display_skiplines", "auto");
 	conf_string_register_default(cfg_context, "display_skipcolumns", "auto");
@@ -1282,12 +1530,12 @@ adv_error advance_video_init(struct advance_video_context* context, adv_conf* cf
 	conf_string_register_default(cfg_context, "display_mode", "auto");
 	conf_int_register_enum_default(cfg_context, "display_color", conf_enum(OPTION_INDEX), 0);
 	conf_bool_register_default(cfg_context, "display_restore", 1);
-	conf_float_register_limit_default(cfg_context, "display_expand", 1.0, 10.0, 1.0);
-	conf_int_register_limit_default(cfg_context, "display_aspectx", 1, 10000, 4);
-	conf_int_register_limit_default(cfg_context, "display_aspecty", 1, 10000, 3);
+	conf_float_register_limit_default(cfg_context, "display_expand", 1.0, 10.0, 1.25);
+	conf_string_register_default(cfg_context, "display_aspect", "auto");
 
 #ifdef USE_SMP
-	conf_bool_register_default(cfg_context, "misc_smp", 0);
+	/* SMP always enabled by default */
+	conf_bool_register_default(cfg_context, "misc_smp", 1);
 #endif
 
 	conf_int_register_enum_default(cfg_context, "sync_resample", conf_enum(OPTION_RESAMPLE), -1);
@@ -1307,7 +1555,7 @@ adv_error advance_video_init(struct advance_video_context* context, adv_conf* cf
 
 static const char* GAME_BLIT_COMBINE_MAX[] = {
 #include "blitmax.h"
-0
+	0
 };
 
 static struct game_startup_struct {
@@ -1315,7 +1563,112 @@ static struct game_startup_struct {
 	int time;
 } GAME_STARTUP[] = {
 #include "startup.h"
-{ 0 }
+	{ 0 }
+};
+
+static const char* GAME_RESAMPLE[] = {
+	/*
+	 * Games using pokey sound chip
+	 *
+	 * At least "starwars" is crashing randomly, and possible others.
+	 * Increasing the buffer size in streams.c avoid the crash,
+	 * but the audio breaks.
+	 */
+
+	/* asteroid */
+	"asteroid",
+	"astdelux",
+	"llander",
+
+	/* atarisys1 */
+	"marble",
+	"peterpak",
+	"indytemp",
+	"roadrunn",
+	"roadblst",
+
+	/* atarisys2 */
+	"paperboy",
+	"720",
+	"ssprint",
+	"csprint",
+	"apb",
+
+	/* atetris */
+	"atetris",
+
+	/* bwidow */
+	"spacduel",
+	"bwidow",
+	"gravitar",
+
+	/* bzone */
+	"bzone",
+	"bradley",
+	"redbaron",
+
+	/* ccastles */
+	"ccastles",
+
+	/* centiped */
+	"centiped",
+	"milliped",
+	"warlords",
+	"ullsdrt",
+
+	/* cloak */
+	"cloak",
+
+	/* cloud9 */
+	"cloud9",
+
+	/* foodf */
+	"foodf",
+
+	/* gauntlet */
+	"gauntlet",
+	"gaunt2",
+	"vindctr2",
+
+	/* irobot */
+	"irobot",
+
+	/* jedi */
+	"jedi",
+
+	/* liberatr */
+	"liberatr",
+
+	/* maxaflex */
+	"maxaflex",
+
+	/* mhavoc */
+	"mhavoc",
+
+	/* missile */
+	"missile",
+
+	/* quantum */
+	"quantum",
+
+	/* runaway */
+	"qwak",
+	"runaway",
+
+	/* starwars */
+	"starwars",
+	"esb",
+
+	/* tempest */
+	"tempest",
+
+	/* toobin */
+	"toobin",
+
+	/* tunhunt */
+	"tunhunt",
+
+	0
 };
 
 void advance_video_config_save(struct advance_video_context* context, const char* section)
@@ -1330,15 +1683,14 @@ void advance_video_config_save(struct advance_video_context* context, const char
 		conf_int_set_if_different(cfg_context, section, "display_magnify", context->config.magnify_factor);
 		conf_int_set_if_different(cfg_context, section, "display_color", context->config.index);
 		if (context->state.game_visible_size_x < context->state.game_used_size_x
-			|| context->state.game_visible_size_y < context->state.game_used_size_y)
-		{
-			if (context->config.skipcolumns>=0) {
+			|| context->state.game_visible_size_y < context->state.game_used_size_y) {
+			if (context->config.skipcolumns >= 0) {
 				char buffer[32];
 				snprintf(buffer, sizeof(buffer), "%d", context->config.skipcolumns);
 				conf_string_set_if_different(cfg_context, section, "display_skipcolumns", buffer);
 			} else
 				conf_string_set_if_different(cfg_context, section, "display_skipcolumns", "auto");
-			if (context->config.skiplines>=0) {
+			if (context->config.skiplines >= 0) {
 				char buffer[32];
 				snprintf(buffer, sizeof(buffer), "%d", context->config.skiplines);
 				conf_string_set_if_different(cfg_context, section, "display_skiplines", buffer);
@@ -1366,7 +1718,7 @@ adv_error advance_video_config_load(struct advance_video_context* context, adv_c
 
 	context->config.game_orientation = mame_game_orientation(option->game);
 
-	for(i=0;GAME_BLIT_COMBINE_MAX[i] != 0;++i)
+	for (i = 0; GAME_BLIT_COMBINE_MAX[i] != 0; ++i)
 		if (mame_is_game_relative(GAME_BLIT_COMBINE_MAX[i], option->game))
 			break;
 	context->config.inlist_combinemax_flag = GAME_BLIT_COMBINE_MAX[i] != 0;
@@ -1383,20 +1735,47 @@ adv_error advance_video_config_load(struct advance_video_context* context, adv_c
 	context->config.triplebuf_flag = conf_bool_get_default(cfg_context, "display_buffer");
 	context->config.stretch = conf_int_get_default(cfg_context, "display_resize");
 	context->config.magnify_factor = conf_int_get_default(cfg_context, "display_magnify");
+	context->config.magnify_size = conf_int_get_default(cfg_context, "display_magnifysize");
 	context->config.adjust = conf_int_get_default(cfg_context, "display_adjust");
 
-	context->config.monitor_aspect_x = conf_int_get_default(cfg_context, "display_aspectx");
-	context->config.monitor_aspect_y = conf_int_get_default(cfg_context, "display_aspecty");
+	context->config.monitor_aspect_x = 0;
+	context->config.monitor_aspect_y = 0;
+	s = conf_string_get_default(cfg_context, "display_aspect");
+	if (strcmp(s, "auto") != 0) {
+		char* e;
+		long x;
+		long y;
+
+		x = strtol(s, &e, 10);
+		while (e && isspace(*e))
+			++e;
+		if (*e == '/') {
+			++e;
+			y = strtol(e, &e, 10);
+			while (e && isspace(*e))
+				++e;
+			if (x > 0 && y > 0 && *e == 0) {
+				context->config.monitor_aspect_x = x;
+				context->config.monitor_aspect_y = y;
+			} else {
+				target_err("Invalid argument '%s' for option 'display_aspect'.\n", s);
+				return -1;
+			}
+		} else {
+			target_err("Invalid argument '%s' for option 'display_aspect'.\n", s);
+			return -1;
+		}
+	}
 
 	s = conf_string_get_default(cfg_context, "display_skiplines");
-	if (strcmp(s, "auto")==0) {
+	if (strcmp(s, "auto") == 0) {
 		context->config.skiplines = -1;
 	} else {
 		context->config.skiplines = atoi(s);
 	}
 
 	s = conf_string_get_default(cfg_context, "display_skipcolumns");
-	if (strcmp(s, "auto")==0) {
+	if (strcmp(s, "auto") == 0) {
 		context->config.skipcolumns = -1;
 	} else {
 		context->config.skipcolumns = atoi(s);
@@ -1419,11 +1798,17 @@ adv_error advance_video_config_load(struct advance_video_context* context, adv_c
 	log_std(("emu:video: orientation ui   %04x\n", option->ui_orientation));
 
 	context->config.combine = conf_int_get_default(cfg_context, "display_resizeeffect");
+	/* on Intel assume a fast machine */
+#if defined(__i386__) || defined(__x86_64__)
+	context->config.combine_max = COMBINE_XBR;
+#else
+	context->config.combine_max = COMBINE_SCALEK;
+#endif
 	context->config.rgb_effect = conf_int_get_default(cfg_context, "display_rgbeffect");
 	context->config.interlace_effect = conf_int_get_default(cfg_context, "display_interlaceeffect");
 	context->config.turbo_speed_factor = conf_float_get_default(cfg_context, "sync_turbospeed");
 	s = conf_string_get_default(cfg_context, "sync_fps");
-	if (strcmp(s, "auto")==0) {
+	if (strcmp(s, "auto") == 0) {
 		context->config.fps_fixed = 0;
 	} else {
 		char* e;
@@ -1439,7 +1824,7 @@ adv_error advance_video_config_load(struct advance_video_context* context, adv_c
 	if (strcmp(s, "none") == 0) {
 		d = 0;
 	} else if (strcmp(s, "auto") == 0) {
-		for(i=0;GAME_STARTUP[i].name != 0;++i)
+		for (i = 0; GAME_STARTUP[i].name != 0; ++i)
 			if (mame_is_game_relative(GAME_STARTUP[i].name, option->game))
 				break;
 		if (GAME_STARTUP[i].name != 0)
@@ -1468,7 +1853,7 @@ adv_error advance_video_config_load(struct advance_video_context* context, adv_c
 	context->config.aspect_expansion_factor = conf_float_get_default(cfg_context, "display_expand");
 
 	s = conf_string_get_default(cfg_context, "display_frameskip");
-	if (strcmp(s, "auto")==0) {
+	if (strcmp(s, "auto") == 0) {
 		context->config.frameskip_auto_flag = 1;
 		context->config.frameskip_factor = 1.0;
 	} else {
@@ -1488,34 +1873,43 @@ adv_error advance_video_config_load(struct advance_video_context* context, adv_c
 #endif
 
 	i = conf_int_get_default(cfg_context, "sync_resample");
-	if (i==1 || i==-1)
+	if (i == -1) {
+		for (i = 0; GAME_RESAMPLE[i] != 0; ++i)
+			if (mame_is_game_relative(GAME_RESAMPLE[i], option->game))
+				break;
+		if (GAME_RESAMPLE[i] != 0)
+			context->config.internalresample_flag = 1; /* internal resampling */
+		else
+			context->config.internalresample_flag = 0; /* emulation resampling */
+	} else if (i == 1) {
 		context->config.internalresample_flag = 1; /* internal resampling */
-	else
+	} else {
 		context->config.internalresample_flag = 0; /* emulation resampling */
+	}
 
 	/* load context->config.monitor config */
 	err = monitor_load(cfg_context, &context->config.monitor);
-	if (err<0) {
+	if (err < 0) {
 		target_err("%s\n", error_get());
 		target_err("Please read the file `install.txt' and `device.txt'.\n");
 		return -1;
 	}
-	if (err==0) {
+	if (err == 0) {
 		char buffer[1024];
 		monitor_print(buffer, sizeof(buffer), &context->config.monitor);
 		log_std(("emu:video: clock %s\n", buffer));
 	}
-	if (err>0) {
+	if (err > 0) {
 		monitor_reset(&context->config.monitor);
 	}
 
 	/* load generate_linear config */
 	err = generate_interpolate_load(cfg_context, &context->config.interpolate);
-	if (err<0) {
+	if (err < 0) {
 		target_err("%s\n", error_get());
 		target_err("Please read the file `install.txt' and `device.txt'.\n");
 		return -1;
-	} else if (err>0) {
+	} else if (err > 0) {
 		if (monitor_clock_check(&context->config.monitor, 7.16E6, 15720, 60)) {
 			/* Arcade Standard Resolution */
 			log_std(("emu:video: default format standard resolution\n"));
@@ -1544,7 +1938,7 @@ adv_error advance_video_config_load(struct advance_video_context* context, adv_c
 		return -1;
 	}
 
-	if (crtc_container_load(cfg_context, &context->config.crtc_bag)!=0) {
+	if (crtc_container_load(cfg_context, &context->config.crtc_bag) != 0) {
 		target_err("%s\n", error_get());
 		target_err("Invalid modeline.\n");
 		return -1;
